#!/usr/bin/env python
# FILE: dnabarcodes_visualize.py
# AUTHOR: Duong Vu
# CREATE DATE: 20 oct 2020

import sys, argparse
import os
if sys.version_info[0] >= 3:
	unicode = str
import numpy
import matplotlib.pyplot as plt
plt.rc('font',size=6)
from mpl_toolkits import mplot3d	
from Bio import SeqIO
import json
import multiprocessing
nproc=multiprocessing.cpu_count()

parser=argparse.ArgumentParser(prog='visualize.py',  
							   usage="%(prog)s [options] -i fastafile -c classificationfilename -rank classificationprank",
							   description='''Script that visualizes the dna sequences based on a given classification. ''',
							   epilog="""Written by Duong Vu duong.t.vu@gmail.com""",
   )

parser.add_argument('-i','--input', required=True, help='the fasta file')
parser.add_argument('-o','--out',default="dnabarcoder", help='The output folder.')
parser.add_argument('-c','--classification', help='the classification file in tab. format.')
#parser.add_argument('-p','--classificationpos', type=int, default=0, help='the classification positions for the prediction.')
parser.add_argument('-rank','--classificationrank', default="", help='the classification rank for coloring the sequences.')
parser.add_argument('-sim','--simfilename', help='The similarity file if exists, othter it will be computed.')
parser.add_argument('-ml','--minalignmentlength', type=int, default=400, help='Minimum sequence alignment length required for BLAST. For short barcode sequences like ITS2 (ITS1) sequences, minalignmentlength should be set to smaller, 50 for instance.')
parser.add_argument('-coord','--coordinates', help='A file containing coordinates of the sequences, computed by LargeVis. If these coordinates will be computed if this file is not given.')
parser.add_argument('-dim','--dimension', type=int, default=3,help='The dimension 2D or 3D for visualization.')
parser.add_argument('-kneigh','--kneigbors', type=int, default=150,help='The k-neighbors number for visualization.')
parser.add_argument('-ms','--minsim', type=float, default=0, help='The minimum similarity for visualization.')
parser.add_argument('-method','--visualizationmethod', default="", help='The visualization method. There are two methods to be selected: dive and plot.')
#parser.add_argument('-lim','--lim', type=int, default=50, help='The lim for visualization.')
parser.add_argument('-size','--size', type=float, default=1, help='The size of the dot.')
parser.add_argument('-n','--numberofdisplayedlabels', type=int, default=5, help='The size of the dot.')
parser.add_argument('-prefix','--prefix',default="", help='The prefix of the output files.')
parser.add_argument('-label','--label',default="", help='The label to display in the figure.')

args=parser.parse_args()
fastafilename= args.input
simfilename=args.simfilename
coordfilename=args.coordinates
classificationfilename=args.classification
#classificationpos=args.classificationpos
rank=args.classificationrank
mincoverage=args.minalignmentlength
minsim=args.minsim
dim=args.dimension
kneigh=args.kneigbors
method=args.visualizationmethod
size=args.size
numberofdisplayedlabels=args.numberofdisplayedlabels
outputpath=args.out
prefix=args.prefix
label=args.label

if not os.path.exists(outputpath):
	os.system("mkdir " + outputpath)	


def GetBase(filename):
	if not ("." in filename):
		return filename
	return filename[:-(len(filename)-filename.rindex("."))] 

#def GetWorkingBase(filename):
#	basename=os.path.basename(filename)
#	if "." in basename:
#		basename=basename[:-(len(basename)-basename.rindex("."))] 
#	path=outputpath + "/" + basename
#	return path

def GetWorkingBase(basename):
	path=outputpath + "/" + basename
	return path

def LoadSim(simfilename,n):
	simmatrix = {} #we use dictionary to reduce the memory constraints 
	simfile = open(simfilename)
	for line in simfile:
		texts=line.rstrip().split(" ")
		i=texts[0]
		j=texts[1]
		if i not in simmatrix.keys():
			simmatrix.setdefault(i, {})
		if j not in simmatrix[i].keys():
			simmatrix[i].setdefault(j, 0)
		if float(texts[2]) > simmatrix[i][j]:
			simmatrix[i][j]=float(texts[2])
	simfile.close()		
	return simmatrix

def SaveSim(simmatrix,simfilename,ms):
	simfile=open(simfilename,"w")
	for i in simmatrix.keys():
		for j in simmatrix[i].keys():
			if simmatrix[i][j] >= ms:
				simfile.write(str(i) + " " + str(j) + " " + str(simmatrix[i][j]) + "\n")
	simfile.close()

def ComputeSim(fastafilename,seqids,mincoverage,minsim):
	#scorematrix = [[0 for x in range(len(seqrecords))] for y in range(len(seqrecords))] 
	simmatrix={}
	for seqid in seqids:
		simmatrix.setdefault(seqid,{})
		simmatrix[seqid][seqid]=1
#	simfilename=GetBase(fastafilename) + ".sim"
#	simfile=open(simfilename,"w")
	#blast
	makedbcommand = "makeblastdb -in " + fastafilename + " -dbtype \'nucl\' " +  " -out db"
	os.system(makedbcommand)
	blastcommand = "blastn -query " + fastafilename + " -db  db -task blastn-short -outfmt 6 -out out.txt -num_threads " + str(nproc)
	if mincoverage >=300:
		blastcommand = "blastn -query " + fastafilename + " -db  db -outfmt 6 -out out.txt -num_threads " + str(nproc)
	os.system(blastcommand)
	
	#read blast output
	blastoutputfile = open("out.txt")
	for line in blastoutputfile:
		if line.rstrip()=="":
			continue
		words = line.split("\t")
#		queryid = words[0].rstrip()
#		refid = words[1].rstrip()
		i = words[0].rstrip()
		j = words[1].rstrip()
#		i = seqids.index(queryid)
#		j = seqids.index(refid)
		pos1 = int(words[6])
		pos2 = int(words[7])
		iden = float(words[2]) 
		sim=float(iden)/100
		coverage=abs(pos2-pos1)
		score=sim
		if coverage < mincoverage:
			score=float(score * coverage)/mincoverage
		if j in simmatrix[i].keys():
			if simmatrix[i][j] < score:
				simmatrix[i][j]=round(score,4)
				simmatrix[j][i]=round(score,4)
		else:
			simmatrix[i][j]=round(score,4)
			simmatrix[j][i]=round(score,4)
#		if scorematrix[i][j] < score:
#			scorematrix[i][j]=score
#			scorematrix[j][i]=score
	os.system("rm out.txt")
	return simmatrix

def ComputeCoordinates(base,simfilename,dim,edgeNo,kneigh):
	coordfilename==GetBase(fastafilename) + ".coord"
	largeViscommand = "LargeVis -fea 0 -input " + simfilename + " -output " + coordfilename + " -outdim " + str(dim) +  " -threads "  + str(nproc) + " -log 1 -samples " + str(edgeNo) + " -neigh " + str(kneigh);
	print(largeViscommand)
	os.system(largeViscommand)
	return coordfilename

def LoadFullClassification(seqids,classificationfilename):
	features=""
	classification=[""]*len(seqids)
	if classificationfilename!=None:
		classificationfile=open(classificationfilename)
		firstline=classificationfile.readline()
		features=firstline.replace("\n","").split("\t")
		if not firstline.startswith("#"):
			seqid = firstline.split("\t")[0].replace(">","")
			if seqid in seqids:  
				classification[seqids.index(seqid)]=firstline.replace("\n","")	        
		for line in classificationfile:
			seqid = line.split("\t")[0].replace(">","")
			if seqid in seqids:
				classification[seqids.index(seqid)]=line.replace("\n","").split("\t")
		classificationfile.close()				
	else:
		features=["seqindex"]
		for i in range(0,len(seqids)):
			classification[i]=[str(i)]
	return features,classification

def LoadClassification(seqids,classificationfilename,classificationpos):
	labels=[""]*len(seqids)
	rank=""
	if classificationfilename!=None:
		classificationfile=open(classificationfilename)
		i=0
		for line in classificationfile:
			seqid = line.split("\t")[0].replace(">","")
			if i==0:
				rank=line.replace("\n","").split("\t")[classificationpos]
			if seqid in seqids:
				labels[seqids.index(seqid)]=line.replace("\n","").split("\t")[classificationpos]
			i=i+1	
		classificationfile.close()				
	else:
		for i in range(0,len(seqids)):
			labels[i]=str(i)
	return labels, rank

def LoadCoordinates(coorfilename):
	coordfile=open(coordfilename)
	firstline=coordfile.readline().rstrip()
	seqNo=int(firstline.split(" ")[0])
	dim=int(firstline.split(" ")[1])
	coordinates={}
	for line in coordfile:
		line=line.rstrip()
		seqid=line.split(" ")[0]
		line=line[line.index(" ")+1:]
		coordinate=line.split(" ")
		coordinates[seqid]=coordinate
	coordfile.close()	
	return seqNo,dim,coordinates

def CreateDiVEFiles(base,seqids,classificationfilename,coorfilename):
	jsonfilename=GetWorkingBase(fastafilename) + ".dive.json"
	divefilename=base + "DiVE/data/data.js"
	features,classification=LoadFullClassification(seqids,classificationfilename)
	seqNo,dim,coordinates=LoadCoordinates(coorfilename)
	coorddict={}
	coorddict["NamesOfProperties"]=features
	i=0
	for seqid in seqids:
		item={}
		item["Coordinates"]=coordinates[seqid]
		item["Properties"]=classification[i]
		coorddict[seqid]=item
		#coorddict[str(i)]=item
		i=i+1
	#write to file
	with open(jsonfilename,"w") as json_file:
		if sys.version_info[0] >= 3:
			json.dump(coorddict,json_file,indent=2)
		else:
			json.dump(coorddict,json_file,encoding='latin1',indent=2)
	f = open(jsonfilename, "r")
	contents = f.readlines()
	f.close()
	contents.insert(0, "const data_all = ")
	f = open(divefilename, "w")
	contents = "".join(contents)
	f.write(contents)
	f.close()
	return divefilename,jsonfilename

def VisualizeUsingDiVE(base,seqids,classificationfilename,coorfilename):
	divefilename,jsonfilename=CreateDiVEFiles(base,seqids,classificationfilename,coorfilename)
	print("The DiVE file is created as " + jsonfilename + ".")
	os.system("firefox " + base + "DiVE/index.html");
	
def Plot(prefix,seqids,coordfilename,classificationfilename,classificationpos,size,output):
	labels,rank = LoadClassification(seqids,classificationfilename,classificationpos)
	seqNo,dim,coordinates=LoadCoordinates(coordfilename)
	all_data = {}
	cmap = plt.get_cmap("tab20c")
	for i in range(0,seqNo):
		all_data.setdefault(labels[i], [0,[]])
	i=0	
	for seqid in seqids:
		coord=coordinates[seqid]
		all_data[labels[i]][0]=all_data[labels[i]][0] + 1
		if dim==2:
			#all_data.setdefault(labels[i], []).append((float(coord[-2]), float(coord[-1])))
			all_data[labels[i]][1].append((float(coord[-2]), float(coord[-1])))
		else:
			all_data[labels[i]][1].append((float(coord[-3]),float(coord[-2]),float(coord[-1])))
		i=i+1	
	#sort all_data based on number of points with a decreasing order
	sorted_data = sorted(all_data.items(), key=lambda x: x[1], reverse=True)
	#colors = plt.cm.rainbow(numpy.linspace(0, 1, len(all_data)))
	n1=min(10,len(all_data))
	colors = plt.cm.tab10(numpy.linspace(0, 1, n1)) #prism
#	if len(colors) >=5:
#		colors=numpy.delete(colors,5,0)#delete the yellow color from the list
#		if len(colors) >=7:
#			colors=numpy.delete(colors,7,0)#delete the yellow color from the list		
#	n1=len(colors)
#	print(n1)
	if n1 < len(all_data):
		#colors1 = plt.cm.rainbow(numpy.linspace(0, 1, (len(all_data)-n1)))
		colors1=cmap(numpy.arange(len(all_data)-n1)) 
		colors=numpy.concatenate((colors,colors1), axis=0)
	 #Set1, primse 
	fig, ax = plt.subplots(figsize=(4,3)) 
	if prefix=="":
		ax.set_title("Sequence distribution \n at the " + rank + " level.",loc='left')
	else:
		ax.set_title(prefix + ": sequence distribution \n at the " + rank + " level.",loc='left')
	labels=[]
	if dim==2:
		# Hide grid lines
		#ax.grid(False)
		# Hide axes ticks
		#ax.set_xticks([])
		#ax.set_yticks([])
		for color, item in zip(colors, sorted_data):
			x = [t[0] for t in item[1][1]]
			y = [t[1] for t in item[1][1]]
			taxon=item[0]
			if taxon=="":
				taxon="unidentified"
			if taxon=="unidentified":
				color="black"	
			label=taxon + " " + str(item[1][0])
			ax.plot(x, y, '.', color = color, markersize = size)	
			labels.append(label)	
	elif dim==3:
		ax = plt.axes(projection='3d')
		ax.set_title(prefix + ": visualization of the sequences \n at the " + rank + " level.", loc='left')
		# Hide grid lines
		#ax.grid(False)
		# Hide axes ticks
		#ax.set_xticks([])
		#ax.set_yticks([])
		#ax.set_zticks([])
		for color, item in zip(colors, sorted_data):
			x = [t[0] for t in item[1][1]]
			y = [t[1] for t in item[1][1]]
			z = [t[2] for t in item[1][1]]
			taxon=item[0]
			if taxon=="":
				taxon="unidentified"
			if taxon=="unidentified":
				color="black"
			label=taxon + " " + str(item[1][0])
			ax.plot3D(x, y, z, '.', color = color, markersize = size)
			labels.append(label)	
			#ax.scatter3D(x, y, z, '.', color = color)
			#ax.axis('off')
	leg=fig.legend(labels[0:numberofdisplayedlabels],loc='best')
	#leg=fig.legend(labels[0:numberofdisplayedlabels],loc='lower left')
	i=0
	for text in leg.get_texts():
		color= colors[i]
		label=text.get_text()
		if label.startswith("unidentified"):
			color="black"
		plt.setp(text, color = color)
		i=i+1
	#plt.xlim(-lim, lim)
    #plt.ylim(-lim, lim)
	plt.rcParams['font.size'] = 6.0
	plt.savefig(output, dpi = 500)
	plt.show()		
	
def GetPosition(classificationfilename,rank):
	pos=-1
	classificationfile=open(classificationfilename)
	header=classificationfile.readline()
	header=header.rstrip()
	classificationfile.close()
	texts=header.split("\t")
	isError=False
	if rank in texts:
		pos=texts.index(rank)
	else:
		print("The rank " + rank + " is not given in the classification." )
		isError=True
	return pos,isError
	
if __name__ == "__main__":
	#load sequences
	if prefix=="":
		prefix=GetBase(os.path.basename(fastafilename))
	base = sys.argv[0][0: sys.argv[0].rindex("/")+1]
	seqrecords=list(SeqIO.parse(fastafilename, "fasta"))
	seqids=[]
	for rec in seqrecords:
		seqids.append(rec.id)
	if coordfilename==None:
		if simfilename==None or simfilename=="":			
			simfilename=GetWorkingBase(prefix)  + ".sim"
			simfilename_minsim=""
			if minsim >0:
				simfilename_minsim=GetWorkingBase(prefix)  + "." + str(int(round(minsim*100,0))) + ".sim"
			else:
				simfilename_minsim=simfilename
		else:
			simfilename_minsim=simfilename
		coordfilename=GetBase(simfilename_minsim)  + "." + str(dim) + ".coord"	
		if not os.path.isfile(coordfilename):
			if not os.path.isfile(simfilename_minsim):
				if not os.path.isfile(simfilename):
					#compute similarity matrix
					print("Computing similarity matrix..")
					simmatrix=ComputeSim(fastafilename,seqids,mincoverage,minsim)
					print("Saving similarity matrix " + simfilename )
					SaveSim(simmatrix,simfilename,0)
					if minsim >0:
						print("Saving similarity matrix " + simfilename_minsim )
						SaveSim(simmatrix,simfilename_minsim,minsim)
				else:		
					print("Loading the exising similarities " + simfilename)
					simmatrix=LoadSim(simfilename,minsim)
					if minsim>0:
						print("Saving similarity matrix " + simfilename_minsim )
						SaveSim(simmatrix,simfilename_minsim,minsim)
			else:
				#use the existing simfilename.
				print("The existing file " + simfilename_minsim + " is used for loading similarity matrix. If you wish to recalculate the similarity matrix, please delete the existing file.")
			#compute coordinates	
			edgeNo=int(len(seqrecords)/100)
			if edgeNo <50:
				edgeNo=50			
			coordfilename=ComputeCoordinates(base,simfilename_minsim,dim,edgeNo,kneigh)
			print("The coordinates are saved in file " + coordfilename)
		else:
			#use the existing simfilename.
			print("The existing file " + coordfilename + " is used for visualization. If you wish to recalculate the coordinates, please delete the existing file.")
	if method.lower()=="dive":
		VisualizeUsingDiVE(base,seqids,classificationfilename,coordfilename)		
	else:
		classificationpos,isError=GetPosition(classificationfilename,rank)
		output=GetWorkingBase(prefix)  + "." + str(classificationpos) + ".visualization.png"
		if label=="":
			label=prefix
		Plot(label,seqids,coordfilename,classificationfilename,classificationpos,size,output)
		print("The visualization is saved in " + output + ".")
			




#!/usr/bin/env python
# FILE: computeVariation.py
# AUTHOR: Duong Vu
# CREATE DATE: 07 June 2019
import sys, argparse
if sys.version_info[0] >= 3:
	unicode = str
import numpy as np
import os
from Bio import SeqIO
import json
import random
import matplotlib.pyplot as plt
plt.rc('font',size=6)
from matplotlib.patches import Polygon
import multiprocessing
nproc=multiprocessing.cpu_count()

parser=argparse.ArgumentParser(prog='computeVariation.py',  
							   usage="%(prog)s [options] -i fastafile -c classificationfilename -ranks classificationranks -ml minalignmentlength  -o output",
							   description='''Script that computes the median and minimum similarity scores within the groups. ''',
							   epilog="""Written by Duong Vu duong.t.vu@gmail.com""",
   )

parser.add_argument('-i','--input', required=True, help='the fasta file to be clustered.')
parser.add_argument('-ml','--minalignmentlength', type=int, default=400, help='Minimum sequence alignment length required for BLAST. For short barcode sequences like ITS2 (ITS1) sequences, minalignmentlength should be set to smaller, 50 for instance.')
parser.add_argument('-o','--out',default="dnabarcoder", help='The output folder.')
parser.add_argument('-c','--classification', help='the classification file in tab. format.')
parser.add_argument('-ranks','--classificationranks', default="", help='the classification ranks to compute variation, separated by ",".')
parser.add_argument('-m','--maxSeqNo', type=int, default=0, help='The maximum number of randomly selected sequences of each class to be computed in the case the groups are too big.')
parser.add_argument('-plt','--plottype', default="boxplot", help='The type of plots. There are two options: boxplot and plot.')
parser.add_argument('-sim','--simfilename', default="", help='The similarity matrix of the sequences if exists.')
parser.add_argument('-prefix','--prefix',default="", help='The prefix of the output files.')
parser.add_argument('-label','--label',default="", help='The label to display in the figure.')
parser.add_argument('-maxSimMatrixSize','--maxSimMatrixSize', type=int, default=20000, help='The maximum number of sequences to load or compute a full similarity matrix. In case the number of sequences is greater than this number, only similarity values greater than 0 will be loaded to avoid memory problems.')


args=parser.parse_args()
referencename= args.input
mincoverage = args.minalignmentlength
classificationfilename=args.classification
jsonvariationfilename =args.out
plottype=args.plottype
simfilename=args.simfilename
prefix=args.prefix
label=args.label


maxSeqNo=0
if args.maxSeqNo !=None:
	maxSeqNo=args.maxSeqNo
outputpath=args.out
if not os.path.exists(outputpath):
	os.system("mkdir " + outputpath)	

def GetBase(filename):
	if not ("." in filename):
		return filename
	return filename[:-(len(filename)-filename.rindex("."))] 

#def GetWorkingBase(filename):
#	basename=os.path.basename(filename)
#	if "." in basename:
#		basename=basename[:-(len(basename)-basename.rindex("."))] 
#	path=outputpath + "/" + basename
#	return path

def GetWorkingBase(basename):
	path=outputpath + "/" + basename
	return path

def LoadClassification(seqIDs,seqrecords,classificationfilename,pos):
	classification=[""]*len(seqIDs)
	classes=[]
	classnames=[]
	if classificationfilename == "":
		return classification,classes,classnames
	records= open(classificationfilename)
	next(records)
	for line in records:
		elements=line.split("\t")
		seqid = elements[0].replace(">","").rstrip()
		classname=""
		if pos < len(elements):
			 classname=elements[pos].rstrip()
		if classname=="" or classname=="unidentified":
			continue 
		if seqid in seqIDs:
			index=seqIDs.index(seqid)
			if classname in classnames:
				classid=classnames.index(classname)
				classes[classid].append(seqrecords[index])
			else:
				classnames.append(classname)
				seqs=[]
				seqs.append(seqrecords[index])
				classes.append(seqs)
	records.close()			
	return classification,classes,classnames

def LoadSim(simfilename):
	simmatrix = {} #we use dictionary to reduce the memory constraints 
	simfile = open(simfilename)
	seqids=[]
	for line in simfile:
		numbers=line.rstrip().split(" ")
		i=numbers[0]
		j=numbers[1]
		seqids.append(i)
		seqids.append(j)
		if i not in simmatrix.keys():
			simmatrix.setdefault(i, {})
		simmatrix[i][j]=float(numbers[2])
	seqids=list(set(seqids))	
	for seqid1 in seqids:
		if not (seqid1 in simmatrix.keys()):
			simmatrix.setdefault(seqid1, {})
			simmatrix[seqid1][seqid1]=1	
		if len(seqids) < args.maxSimMatrixSize: #load full matrix	
			for seqid2 in seqids:
				if not seqid2 in simmatrix[seqid1].keys():
					#simmatrix[seqid1].setdefault(seqid2,0)
					simmatrix[seqid1][seqid2]=0
	simfile.close()		
	return simmatrix

def GetSeqIndex(seqname,seqrecords):
	i=0
	for seqrecord in seqrecords:
		if (seqname == seqrecord.id):
			return i
		i = i + 1
	return -1

def ComputeSim(fastafilename,seqrecords,mincoverage):
	blastoutput = fastafilename + ".blast.out"		
	blastdb=fastafilename + ".db"		
	#blast
	print("Comparing the sequences of " + fastafilename + " using Blast...")
	makedbcommand = "makeblastdb -in " + fastafilename + " -dbtype \'nucl\' " +  " -out " + blastdb
	print(makedbcommand)
	os.system(makedbcommand)
	blastcommand = "blastn -query " + fastafilename + " -db  " + blastdb + " -task blastn-short -outfmt 6 -out " + blastoutput + " -num_threads " + str(nproc)
	if mincoverage >=400:
		blastcommand = "blastn -query " + fastafilename + " -db " + blastdb + " -outfmt 6 -out " + blastoutput + " -num_threads " + str(nproc)
	print(blastcommand)
	os.system(blastcommand)
	if not os.path.exists(blastoutput):
		print("Cannot compare the sequences of " + fastafilename + " using Blast...")
		logfile=open(GetWorkingBase((os.path.basename(args.input))) + ".predict.log","w")
		logfile.write("Cannot compare the sequences of " + fastafilename + " using Blast...")
		logfile.write("Make BLAST database command: " + makedbcommand + "\n")
		logfile.write("No output for the BLAST command: " + blastcommand + "\n")
		logfile.write("Please rerun prediction for " + os.path.basename(fastafilename) + ".")
		logfile.close()
		return {}
	print("Reading Blast results of " + fastafilename + "...")
	simmatrix={}
	for seqid1 in seqrecords.keys():
		simmatrix.setdefault(seqid1,{})
		for seqid2 in seqrecords.keys():
			if seqid1==seqid2:
				simmatrix[seqid1][seqid2]=1
			elif len(seqrecords.keys()) < args.maxSimMatrixSize: #load full matrix	
				simmatrix[seqid1][seqid2]=0
	#read blast output
	blastoutputfile = open(blastoutput)
	score=0
	for line in blastoutputfile:
		if line.rstrip()=="":
			continue
		words = line.split("\t")
		i = words[0].rstrip()
		j = words[1].rstrip()
		pos1 = int(words[6])
		pos2 = int(words[7])
		iden = float(words[2]) 
		sim=float(iden)/100
		coverage=abs(pos2-pos1)
		score=sim
		if coverage < mincoverage:
			score=float(score * coverage)/mincoverage
		if len(seqrecords.keys()) < args.maxSimMatrixSize: #the full sim matrix has been loaded 
			if simmatrix[i][j] < score:
				simmatrix[i][j]=round(score,4)
				simmatrix[j][i]=round(score,4)
		else:		
			if j in simmatrix[i].keys():
				if simmatrix[i][j] < score:
					simmatrix[i][j]=round(score,4)
					simmatrix[j][i]=round(score,4)
			else:
				simmatrix[i][j]=round(score,4)
				simmatrix[j][i]=round(score,4)	
		#simmatrix[j][i]=score
	os.system("rm " + blastoutput)
	os.system("rm " + blastdb + "*")
	#os.system("rm " + blastdb + ".*")
	return simmatrix

def ComputeVariation(reffilename,mincoverage,simmatrix):
	#load sequeces from the fasta files
	records = SeqIO.to_dict(SeqIO.parse(reffilename, "fasta"))
	keys=list(records.keys())
	scorelist=[]
	check=False
	if simmatrix!={}:
		check=True
		for i in range(0,len(keys)-2):
			if not keys[i] in simmatrix.keys():
				check==False
				break
			for j in range(i+1,len(keys)-1):
				if i!=j:
					if not keys[j] in simmatrix[keys[i]].keys():
						check==False
						break
					scorelist.append(simmatrix[keys[i]][keys[j]])	
	if check==False:
		scorematrix=ComputeSim(reffilename,records,mincoverage)
		for i in range(0,len(keys)-2):
			for j in range(i+1,len(keys)-1):
				if i!=j:
					scorelist.append(scorematrix[keys[i]][keys[j]])	
	threshold=1
	minthreshold=1		
	if len(scorelist) >0:
		x=np.array(scorelist)
		minthreshold=round(float(np.min(x)),4)
		threshold=round(float(np.median(x)),4)
	return threshold,minthreshold

def ComputeVariations(variationfilename,classes,classnames,mincoverage,simmatrix):
	#create json dict
	variations={}
	i=0
	for taxonname in classnames:
		threshold=0
		minthreshold=0
		sequences=classes[i]
		if len(sequences) >0:
			if maxSeqNo==0 or (len(sequences) < maxSeqNo):
				fastafilename=taxonname.replace(" ","_") + ".fasta"
				SeqIO.write(sequences,fastafilename,"fasta")
				threshold,minthreshold=ComputeVariation(fastafilename,mincoverage,simmatrix)
				os.system("rm " + fastafilename)
			else:
				threshold,minthreshold=EvaluateVariation(taxonname,sequences,mincoverage)
			currentvariation=[threshold,minthreshold,len(sequences)]
			variations[taxonname]=currentvariation
		i=i+1	
	#write to file
	with open(variationfilename,"w") as json_file:
		if sys.version_info[0] >= 3:
			json.dump(variations,json_file,indent=2)	
		else:
			json.dump(variations,json_file,encoding='latin1',indent=2)	
	return variations

def EvaluateVariation(taxonname,sequences,mincoverage,simmatrix):
	selectedindexes=random.sample(range(0, len(sequences)), k=maxSeqNo)
	selectedsequences=[]
	for index in selectedindexes:
		selectedsequences.append(sequences[index])
	fastafilename=taxonname.replace(" ","_") + ".fasta"
	SeqIO.write(selectedsequences,fastafilename,"fasta")
	threshold,minthreshold=ComputeVariation(fastafilename,mincoverage,simmatrix)
	return threshold,minthreshold
		
def IndexSequences(filename):
	indexedfilename = GetBase(filename) + ".indexed.fasta"
	fastafile = open(filename)
	indexedfile = open(indexedfilename, "w")
	i=0
	for line in fastafile:
		if line.startswith('>'):
			indexedfile.write(">" + str(i) + "|" + line.rstrip()[1:] + "\n")
			i=i+1
		else:
			indexedfile.write(line)    
	fastafile.close()
	indexedfile.close()
	return indexedfilename

def SaveVariationInTabFormat(output,variation):
	outputfile=open(output,"w")
	outputfile.write("Taxonname\tMedian similarity score\tMin similarity score\tNumber of sequences\n")	
	for classname in variation.keys():
		threshold=variation[classname][0]
		minthreshold=variation[classname][1]
		seqno=variation[classname][2]
		outputfile.write(classname + "\t" + str(threshold) + "\t" + str(minthreshold) + "\t" + str(seqno) + "\n")
	outputfile.close()
	
def Plot(datasetname,figoutput,variations,rank,displayed):
	#sort variations based on median thresholds with decreasing order
	sorted_variations = sorted(variations.items(), key=lambda x: x[1][0], reverse=True)
	thresholds=[]
	minthresholds=[]
	seqnos=[]
	for item in sorted_variations:
		threshold=item[1][0]
		minthreshold=item[1][1]
		seqno=item[1][2]
		thresholds.append(threshold)
		minthresholds.append(minthreshold)
		seqnos.append(seqno)
	median_median=round(np.median(np.array(thresholds))	,4)
	median_min=round(np.median(np.array(minthresholds))	,4)
	x = np.arange(len(seqnos))
	fig, ax2 = plt.subplots(figsize=(3,3))
	ax = ax2.twinx()  # instantiate a second axes that shares the same x-axi
	ax2.set_ylabel('Number of sequences')  # we already handled the x-label with ax1
	ax2.plot(x, np.array(seqnos), color='g')
	ax.set_title(datasetname + ": median and minimum similarity scores of the " + rank.lower() )
	ax2.set_xlabel("Group index")
	ax.set_ylabel('Similarity score')
	#plt.plot(x, np.array(thresholds), 'r--', x,minthresholds, 'bs') #green 'g^'
	ax.plot(x, np.array(thresholds), 'b--', label=  'Median. Median score' + str(median_median))
	ax.plot(x, np.array(minthresholds), 'rs', label='Min. Median score: ' + str(median_min))
	plt.legend()
	plt.tight_layout()
	plt.rcParams['font.size'] = 6.0
	plt.savefig(figoutput, dpi = 500)
	if displayed==True:
		plt.show()

def PlotAll(datasetname,figoutput,variationlist,labels):
	data=[]
	for variations in variationlist:
		sorted_variations = sorted(variations.items(), key=lambda x: x[1][0], reverse=True)
		thresholds=[]
		for item in sorted_variations:
			threshold=item[1][0]
			#minthreshold=item[1][1]
			#seqno=item[1][2]
			thresholds.append(threshold)
			#minthresholds.append(minthreshold)
			#seqnos.append(seqno)
		data.append(thresholds)
	colors = plt.cm.Set1(np.linspace(0, 1,len(data)))	
	fig, ax = plt.subplots(figsize=(3,3))
	if datasetname=="":
		ax.set_title("Median similarity scores of all groups")
	else:
		ax.set_title(datasetname + ": median similarity scores of all groups")
	ax.set_xlabel("Group index")
	ax.set_ylabel('Median similarity score')
	#plt.plot(x, np.array(thresholds), 'r--', x,minthresholds, 'bs') #green 'g^'
	k=0
	for thresholds in data:
		x = np.arange(len(thresholds))
		median_median=np.median(np.array(thresholds))
		ax.plot(x, np.array(thresholds), color=colors[k], label=  labels[k] + '. Median ' + str(median_median))
		k=k+1
	plt.legend()
	plt.tight_layout()
	plt.rcParams['font.size'] = 6.0
	plt.savefig(figoutput, dpi = 500)
	plt.show()
	
def BoxPlot(datasetname,figoutput,variations,rank,displayed):
	#sort variations based on median thresholds with decreasing order
	sorted_variations = sorted(variations.items(), key=lambda x: x[1][0], reverse=True)
	thresholds=[]
	minthresholds=[]
	seqnos=[]
	for item in sorted_variations:
		threshold=item[1][0]
		minthreshold=item[1][1]
		seqno=item[1][2]
		thresholds.append(threshold)
		minthresholds.append(minthreshold)
		seqnos.append(seqno)
	labels=["Median", "Min"]	
	data=[np.array(thresholds),np.array(minthresholds)]
#	fig, ax = plt.subplots(figsize=(10, 6))
#	#fig.canvas.set_window_title('Variation')
#	fig.subplots_adjust(left=0.075, right=0.95, top=0.9, bottom=0.25)
	fig, ax = plt.subplots(figsize=(3,3))
	box_colors = ['b','r']#['darkkhaki', 'royalblue']
	bp = ax.boxplot(data, notch=0, sym='+', vert=1, whis=1.5)
	plt.setp(bp['boxes'], color='black')
	plt.setp(bp['whiskers'], color='black')
	plt.setp(bp['fliers'], color='red', marker='+')
	
	ax.yaxis.grid(True, linestyle='-', which='major', color='lightgrey',
               alpha=0.5)

	# Hide these grid behind plot objects
	ax.set_axisbelow(True)
	if datasetname=="":
		ax.set_title('Median and min. similarity scores of ' + rank)
	else:
		ax.set_title(datasetname + ': median and min. similarity scores of ' + rank)
	#ax.set_xlabel('')
	ax.set_ylabel('Similarity score')
	num_boxes=len(data)
	medians=np.empty(num_boxes)
	for i in range(num_boxes):
		box=bp['boxes'][i]
		boxX=[]
		boxY=[]
		for j in range(5):
			boxX.append(box.get_xdata()[j])
			boxY.append(box.get_ydata()[j])
		box_coords = np.column_stack([boxX, boxY])	
		# Fill in the color
		ax.add_patch(Polygon(box_coords, facecolor=box_colors[i % 2]))
		med = bp['medians'][i]
		medianX=[]
		medianY=[]
		for j in range(2):
			medianX.append(med.get_xdata()[j])
			medianY.append(med.get_ydata()[j])
		medians[i] = medianY[0]
		#plot the average value
		ax.plot(np.average(med.get_xdata()), np.average(data[i]),color='w', marker='*', markeredgecolor='k')
	#add labels	
	ax.set_xticklabels(np.array(labels))	
	#add median values 
	upper_labels = [str(np.round(s, 4)) for s in medians]
	pos = np.arange(num_boxes) + 1
	k=0
	for tick, label in zip(range(num_boxes), ax.get_xticklabels()):
		ax.text(pos[tick], 0.97, upper_labels[tick], transform=ax.get_xaxis_transform(), horizontalalignment='center', size='x-small', color=box_colors[k])
		k=k+1
	#plt.legend()
	plt.tight_layout()
	plt.rcParams['font.size'] = 6.0
	plt.savefig(figoutput, dpi = 500)
	if displayed==True:
		plt.show()
		
def BoxPlotAll(datasetname,figoutput,variationlist,labels):
	data=[]
	labels2=[]
	colors=[]
	i=0
	for variations in variationlist:
		sorted_variations = sorted(variations.items(), key=lambda x: x[1][0], reverse=True)
		thresholds=[]
		minthresholds=[]
		for item in sorted_variations:
			threshold=item[1][0]
			minthreshold=item[1][1]
			#seqno=item[1][2]
			thresholds.append(threshold)
			minthresholds.append(minthreshold)
			#seqnos.append(seqno)
		data.append(thresholds)
		data.append(minthresholds)
		labels2.append("Median_" + labels[i])
		labels2.append("Min_" + labels[i])
		colors.append('b')
		colors.append('r')
		i=i+1
#	fig, ax = plt.subplots(figsize=(10, 6))
#	#fig.canvas.set_window_title('Variation')
#	fig.subplots_adjust(left=0.075, right=0.95, top=0.9, bottom=0.25)
	#box_colors = ['r','b']#['darkkhaki', 'royalblue']
	fig, ax = plt.subplots(figsize=(4,3))
	bp = ax.boxplot(data, notch=0, sym='+', vert=1, whis=1.5)
	plt.setp(bp['boxes'], color='black')
	plt.setp(bp['whiskers'], color='black')
	plt.setp(bp['fliers'], color='red', marker='+')
	
	ax.yaxis.grid(True, linestyle='-', which='major', color='lightgrey', alpha=0.5)
	# Hide these grid behind plot objects
	ax.set_axisbelow(True)
	ax.set_title(datasetname + ': median and min. similarity scores of all groups')
	#ax.set_xlabel('')
	ax.set_ylabel('Similarity score')
	num_boxes=len(data)
	#colors = plt.cm.Set1(np.linspace(0, 1,num_boxes))
	medians=np.empty(num_boxes)
	for i in range(num_boxes):
		box=bp['boxes'][i]
		boxX=[]
		boxY=[]
		for j in range(5):
			boxX.append(box.get_xdata()[j])
			boxY.append(box.get_ydata()[j])
		box_coords = np.column_stack([boxX, boxY])	
		# Fill in the color
		ax.add_patch(Polygon(box_coords, facecolor=colors[i]))
		med = bp['medians'][i]
		medianX=[]
		medianY=[]
		for j in range(2):
			medianX.append(med.get_xdata()[j])
			medianY.append(med.get_ydata()[j])
		medians[i] = medianY[0]
		#plot the average value
		ax.plot(np.average(med.get_xdata()), np.average(data[i]),color='w', marker='*', markeredgecolor='k')	
	#add labels	
	ax.set_xticklabels(np.array(labels2), rotation=90)	
	#add median values 
	upper_labels = [str(np.round(s, 4)) for s in medians]
	pos = np.arange(num_boxes) + 1
	k=0
	for tick, label in zip(range(num_boxes), ax.get_xticklabels()):
		ax.text(pos[tick], 0.97, upper_labels[tick], transform=ax.get_xaxis_transform(), horizontalalignment='center', size='x-small', color=colors[k])
		k=k+1
	#plt.legend()
	plt.tight_layout()
	plt.rcParams['font.size'] = 6.0
	plt.savefig(figoutput, dpi = 500)
	plt.show()		

def GetPositionList(classificationfilename,ranks):
	ranklist=[]	
	if "," in ranks:
		ranklist=ranks.split(",")
	elif ranks !="":
		ranklist.append(ranks)
	positionlist=[]
	classificationfile=open(classificationfilename)
	header=classificationfile.readline()
	header=header.rstrip()
	classificationfile.close()
	texts=header.split("\t")
	isError=False
	for rank in ranklist:
		if rank in texts:
			pos=texts.index(rank)
			positionlist.append(pos)
		else:
			print("The rank " + rank + " is not given in the classification." )
			isError=True
	return positionlist,ranklist,isError

##############################################################################
# MAIN
##############################################################################
path=sys.argv[0]
path=path[:-(len(path)-path.rindex("/")-1)]

if prefix=="":
	prefix=GetBase(os.path.basename(referencename))
	
positionlist,ranklist,isError=GetPositionList(classificationfilename,args.classificationranks)	
if isError==True:
	sys.exit()
displayed=False
if len(positionlist)==1:
	displayed=True
#load similarity matrix
simmatrix={}	
if os.path.exists(simfilename):
	print("Loading similarity matrix " + simfilename)
	simmatrix=LoadSim(simfilename)	
#load train seq records
referencerecords =  list(SeqIO.parse(referencename, "fasta"))
referenceIDs=[]
for seq in referencerecords:
	referenceIDs.append(seq.id)
variationlist=[]
labels=[]
i=0
for classificationposition in positionlist:
	rank=ranklist[i]
	jsonvariationfilename = GetWorkingBase(prefix) + "." + str(classificationposition) + ".variation"
	figoutput=GetBase(jsonvariationfilename) + ".variation.png" 
	#Load classes, classification:
	referenceclassification,classes,classnames= LoadClassification(referenceIDs,referencerecords,classificationfilename, classificationposition)
	rank=rank.lower()
	if rank=="":
		rank="groups at position " + str(classificationposition)
	variations={}
	if not os.path.exists(jsonvariationfilename):
		variations=ComputeVariations(jsonvariationfilename,classes,classnames,mincoverage,simmatrix)
	else:
		print("The variation file " + jsonvariationfilename + " exists. Please delete the file if you wish to recalculate the variation.")
		with open(jsonvariationfilename) as variation_file:
			variations = json.load(variation_file)
		SaveVariationInTabFormat(jsonvariationfilename + ".txt",variations)
		print("The variations are saved in the json file  " + jsonvariationfilename + " and tab file " + jsonvariationfilename + ".txt. The figure is saved in " + figoutput + "."  )
		#plot
#		if plottype=="plot":
#			Plot(prefix,figoutput,variations,rank,displayed)
#		else:	
#			BoxPlot(prefix,figoutput,variations,rank,displayed)
	variationlist.append(variations)
	labels.append(rank)	
	i=i+1	
if label=="":
	label=prefix	
if len(positionlist)>1:
	jsonvariationfilename = GetWorkingBase(prefix) + ".variation"
	figoutput=jsonvariationfilename + ".png" 
if plottype=="plot":
	PlotAll(label,figoutput,variationlist,labels)
else:	
	BoxPlotAll(label,figoutput,variationlist,labels)
print("All variations and theirs figure are saved in file " + jsonvariationfilename + " and " + figoutput + ".")
			





#!/usr/bin/env python
# FILE: addFeaturesToTaxa.py, requiring Python 2.7 if classification file contains non-ascii characters
# AUTHOR: Duong Vu
# CREATE DATE: 07 June 2019
import sys, argparse
import numpy as np
import os
from Bio import SeqIO
import json
import multiprocessing
nproc=multiprocessing.cpu_count()
#from keras.utils import np_utils

parser=argparse.ArgumentParser(prog='addFeaturesTotTaxa.py',  
							   usage="%(prog)s [options] -i taxafile -ip taxapositionintheinputfile -f featurefile -fp featurepositionsinthefeaturefile -o output",
							   description='''Script that adds features of the taxa given in the features file to the taxa in the taxafile. ''',
							   epilog="""Written by Duong Vu duong.t.vu@gmail.com""",
   )

parser.add_argument('-i','--input', required=True, help='the input file containing taxa in a tab limited format.')
parser.add_argument('-ip','--inputpos', required=True, type =int, help='the position in the input file containing the taxa.')
parser.add_argument('-o','--out', help='the output file.') 
parser.add_argument('-f','--features', required=True, help='the features file containing features associated with some taxa in a tab. limited format.')
parser.add_argument('-fp','--featurepositions', required=True, default="", help='the positions of the feature in the feature file.')
parser.add_argument('-unique','--unique', default="", help='If unique=="" then the most dominant feature is selected, otherwise all features are selected.')

args=parser.parse_args()
inputfilename= args.input
featurefilename=args.features
pos=args.inputpos
featurepositions=args.featurepositions
unique=args.unique
output=args.out

def LoadFeatures(featurefilename,featurepositions):
	featuredict={}
	featurefile=open(featurefilename,errors="ignore")
	header=next(featurefile)
	poslist=[]
	if "," in featurepositions:
		texts=featurepositions.split(",")
		for text in texts:
			poslist.append(int(text))
	else:
		poslist.append(int(featurepositions))
	featurelabellist=[]
	for featurepos in poslist:
		featurelabel=header.split("\t")[featurepos]
		featurelabel=featurelabel.rstrip()
		featurelabellist.append(featurelabel)
	for line in featurefile:
		texts=line.split("\t")
		for text in texts:
			text=text.rstrip()
			if text=="":
				continue
			if not (text in featuredict.keys()):
				featuredict.setdefault(text,{})
			subfeaturedict=featuredict[text]
			for featurepos in poslist:
				if not (featurepos in subfeaturedict.keys()):
					subfeaturedict.setdefault(featurepos,{})
				feature=""	
				if featurepos < len(texts):	
					feature=texts[featurepos].rstrip()
				if feature!="":
					if (not (feature in subfeaturedict[featurepos].keys())):
						subfeaturedict[featurepos].setdefault(feature,0)
					subfeaturedict[featurepos][feature]=subfeaturedict[featurepos][feature]+1
	featurefile.close()				
	return featuredict,featurelabellist,poslist

#####MAIN
featuredict,featurelabellist,poslist=LoadFeatures(featurefilename,featurepositions)	
outputfile=open(output,"w")
inputfile=open(inputfilename,errors="ignore")
header=next(inputfile)
header=header.replace("\n","")
for label in featurelabellist:
	header=header + "\t" + label 
outputfile.write(header + "\n")
for line in inputfile:
	if pos>=len(line.split("\t")):
		continue
	taxon=line.split("\t")[pos].rstrip()
	line=line.replace("\n","")
	if taxon in featuredict.keys():
		subfeaturedict=featuredict[taxon]
		for featurepos in poslist:
			feature=""
			featurelist=subfeaturedict[featurepos]
			if unique=="":
				i=0
				for f in featurelist.keys():
					feature=feature + f
					if i<len(featurelist.keys())-1:
						feature=feature+","
			else:
				m=0
				for f in featurelist.keys():
					if m<featurelist[f]:
						m=featurelist[f]
				for f in featurelist.keys():
					if m==featurelist[f]:
						feature=feature + f + ","
				feature=feature[:-1]		
			line=line + "\t" + feature 
	outputfile.write(line + "\n")
inputfile.close()
outputfile.close()	
print("The output is saved in file " + output + ".")	




#!/usr/bin/env python
# FILE: computeLocalCutoffs.py
# AUTHOR: Duong Vu
# CREATE DATE: 07 June 2021
import sys
if sys.version_info[0] >= 3:
	unicode = str
import os, argparse
from sklearn.metrics import precision_recall_fscore_support
from sklearn.metrics import cohen_kappa_score
from sklearn.metrics import matthews_corrcoef
from sklearn.metrics import confusion_matrix
from sklearn.metrics import accuracy_score
import json
from Bio import SeqIO
import random
import multiprocessing
parser=argparse.ArgumentParser(prog='computeBestCutoffs.py',  
							   usage="%(prog)s [options] -i cutoffs -c classificationfile -o output",
							   description='''Script that computes best cutoffs of the taxa given in the cutoffs and classification file for sequence identification at different taxonomic levels.''',
							   epilog="""Written by Duong Vu duong.t.vu@gmail.com""",
   )

parser.add_argument('-i','--input',required=True, help='the cutoffs file')
parser.add_argument('-o','--out', default="dnabarcoder", help='The output folder.')
parser.add_argument('-c','--classification', required=True, help='the classification file in tab. format.')
parser.add_argument('-cutoffs','--cutoffs', help='The json file containing the cutoffs to assign the sequences to the predicted taxa.')
#parser.add_argument('-minSeqNo','--minseqno', type=int, default=50, help='the minimum number of sequences for using the predicted cut-offs to assign sequences. Only needed when the cutoffs file is given.')
#arser.add_argument('-minGroupNo','--mingroupno', type=int, default=10, help='the minimum number of groups for using the predicted cut-offs to assign sequences. Only needed when the cutoffs file is given.')
parser.add_argument('-prefix','--prefix', help='the prefix of output filenames')
parser.add_argument('-savebestcutoffsascutoffs','--savebestcutoffsascutoffs', default="yes", help='the prefix of output filenames')

args=parser.parse_args()
classificationfilename=args.classification
cutoffsfilename=args.input
outputpath=args.out
prefix=args.prefix
#minGroupNo=args.mingroupno
#minSeqNo=args.minseqno

if not os.path.exists(outputpath):
	os.system("mkdir " + outputpath)

nproc=multiprocessing.cpu_count()

def GetBase(filename):
	return filename[:-(len(filename)-filename.rindex("."))]
	
def GetWorkingBase(filename):
	basename=os.path.basename(filename)
	if "." in basename:
		basename=basename[:-(len(basename)-basename.rindex("."))] 
	path=outputpath + "/" + filename
	return path

def GetRankTaxonomicClassification(level,classification):
	species=classification.split(";")[6].replace("s__","")
	genus=classification.split(";")[5].replace("g__","")
	family=classification.split(";")[4].replace("f__","")
	order=classification.split(";")[3].replace("o__","")
	bioclass=classification.split(";")[2].replace("c__","")
	phylum=classification.split(";")[1].replace("p__","")
	kingdom=classification.split(";")[0].replace("k__","")
	newclassification=""
	rank=""
	if level >=0 and kingdom!="unidentified":
		newclassification="k__" + kingdom 
		rank="kingdom"
	if level >=1 and phylum!="unidentified":
		newclassification="k__" + kingdom +";p__"+ phylum
		rank="phylum"
	if level >=2 and bioclass!="unidentified":
		newclassification="k__" + kingdom +";p__"+phylum +";c__"+bioclass
		rank="class"
	if level >=3 and order!="unidentified":
		newclassification="k__" + kingdom +";p__"+phylum +";c__"+bioclass +";o__"+ order 
		rank="order"
	if level >=4 and family!="unidentified":
		newclassification="k__" + kingdom +";p__"+phylum +";c__"+bioclass +";o__"+ order+";f__"+family 
		rank="family"
	if level >=5 and genus!="unidentified":
		newclassification="k__" + kingdom +";p__"+phylum +";c__"+bioclass +";o__"+ order+";f__"+family + ";g__"+ genus 
		rank="genus"
	if level >=6 and species!="unidentified":
		newclassification="k__" + kingdom +";p__"+phylum +";c__"+bioclass +";o__"+ order+";f__"+family + ";g__"+ genus+";s__"+species 		
		rank="species"
	return newclassification,rank

def GetTaxonomicClassification(level,header,texts):
	classification=""
	p_s=len(texts)
	p_g=len(texts)
	p_f=len(texts)
	p_o=len(texts)
	p_c=len(texts)
	p_p=len(texts)
	p_k=len(texts)
	i=0
	for text in header.split("\t"):
		text=text.rstrip()
		if text.lower()=="species":
			p_s=i
		elif text.lower()=="genus":
			p_g=i	
		elif text.lower()=="family":
			p_f=i	
		elif text.lower()=="order":
			p_o=i	
		elif text.lower()=="class":
			p_c=i	
		elif text.lower()=="phylum":
			p_p=i	
		elif text.lower()=="kingdom":
			p_k=i	
		i=i+1 
	species="unidentified"
	genus="unidentified"
	family="unidentified"
	order="unidentified" 
	bioclass="unidentified"
	phylum="unidentified"
	kingdom="unidentified"
	if p_s< len(texts):
		species=texts[p_s].rstrip()
	if p_g< len(texts):
		genus=texts[p_g].rstrip()
	if p_f< len(texts):
		family=texts[p_f].rstrip()
	if p_o< len(texts):
		order=texts[p_o].rstrip()
	if p_c< len(texts):
		bioclass=texts[p_c].rstrip()
	if p_p< len(texts):
		phylum=texts[p_p].rstrip()
	if p_k< len(texts):
		kingdom=texts[p_k].rstrip()
	taxonname=""
	rank=""
	if level <7 and kingdom!="unidentified":
		taxonname=kingdom
		rank="kingdom"
		classification="k__" + kingdom +";p__unidentified;c__unidentified;o__unidentified;f__unidentified;g__unidentified;s__unidentified"	
	if level <6 and phylum!="unidentified":
		taxonname=phylum
		rank="phylum"
		classification="k__" + kingdom +";p__"+phylum +";c__unidentified;o__unidentified;f__unidentified;g__unidentified;s__unidentified"
	if level <5 and bioclass!="unidentified":
		taxonname=bioclass
		rank="class"
		classification="k__" + kingdom +";p__"+phylum +";c__"+bioclass+";o__unidentified;f__unidentified;g__unidentified;s__unidentified"
	if level <4 and order!="unidentified":
		taxonname=order
		rank="order"
		classification="k__" + kingdom +";p__"+phylum +";c__"+bioclass +";o__"+ order + ";f__unidentified;g__unidentified;s__unidentified"
	if level <3 and family!="unidentified":
		taxonname=family
		rank="family"
		classification="k__" + kingdom +";p__"+phylum +";c__"+bioclass +";o__"+ order+";f__"+family +";g__unidentified;s__unidentified"
	if level <2 and genus!="unidentified":
		taxonname=genus
		rank="genus"
		classification="k__" + kingdom +";p__"+phylum +";c__"+bioclass +";o__"+ order+";f__"+family + ";g__"+ genus + "s__unidentified"
	if level <1 and species!="unidentified":
		taxonname=species
		rank="species"
		classification="k__" + kingdom +";p__"+phylum +";c__"+bioclass +";o__"+ order+";f__"+family + ";g__"+ genus+";s__"+species 	
	return classification,taxonname,rank

def LoadClassification(classificationfilename):
	classificationdict={}
	if classificationfilename == "":
		return {}
	classificationfile= open(classificationfilename)
	header=next(classificationfile)
	for line in classificationfile:
		texts=line.split("\t")
		classification,taxonname,rank=GetTaxonomicClassification(0,header,texts)
		taxonnames=classification.split(";")
		level=0
		for taxonname in taxonnames:
			taxonname=taxonname.split("__")[1]
			if taxonname=="unidentified" or taxonname=="":
				level=level+1
				continue
			newclassification,rank=GetRankTaxonomicClassification(level,classification)	
			if taxonname in classificationdict.keys():
				if len(newclassification.replace("unidentified","")) > len(classificationdict[taxonname]):
					classificationdict[taxonname]["classification"]=newclassification
					classificationdict[taxonname]["rank"]=rank
			else:
				item={}
				item["classification"]=newclassification
				item["rank"]=rank
				classificationdict.setdefault(taxonname,item)		
			level=level+1	
	classificationfile.close()	
	return classificationdict

def GetLevel(rank):
	level=-1
	if rank=="species":	
		level=6
	elif rank=="genus":	
		level=5
	elif rank=="family":	
		level=4
	elif rank=="order":	
		level=3
	elif rank=="class":	
		level=2
	elif rank=="phylum":
		level=1
	elif rank=="kingdom":
		level=0	
	return level

def GetHigherTaxa(rank,classification):
	highertaxa=[]
	if classification=="":
		return highertaxa
	taxa=classification.split(";")
	species=""
	genus=""
	family=""
	order=""
	bioclass=""
	phylum=""
	kingdom=""
	for taxon in taxa:
		if taxon.startswith("k__"):
			kingdom=taxon.replace("k__","")
		if taxon.startswith("p__"):
			phylum=taxon.replace("p__","")	
		if taxon.startswith("c__"):
			bioclass=taxon.replace("c__","")	
		if taxon.startswith("o__"):
			order=taxon.replace("o__","")	
		if taxon.startswith("f__"):
			family=taxon.replace("f__","")	
		if taxon.startswith("g__"):
			genus=taxon.replace("g__","")	
		if taxon.startswith("s__"):
			species=taxon.replace("s__","")	
	level=GetLevel(rank)	
	if level >=6 and species!="" and species!="unidentified":
		highertaxa.append(species)
	if level >=5 and genus!="" and genus!="unidentified":
		highertaxa.append(genus)
	if level >=4 and family!="" and family!="unidentified":
		highertaxa.append(family)	
	if level >=3 and order!="" and order!="unidentified":
		highertaxa.append(order)	
	if level >=2 and bioclass!="" and bioclass!="unidentified":
		highertaxa.append(bioclass)	
	if level >=1 and phylum!="" and phylum!="unidentified":
		highertaxa.append(phylum)
	if level >=0 and kingdom!="" and kingdom!="unidentified":
		highertaxa.append(kingdom)
	return highertaxa

def GetCutoffAndConfidence(rank,classification,cutoffs):
	highertaxa=GetHigherTaxa(rank,classification)
	highertaxa.append("All") 
	#seqno=0
	#groupno=0
	datasets=cutoffs[rank]
	maxconfidence=0
	bestcutoff=0
	bestminalignmentlength=0
	besttaxon=""
	for highertaxonname in highertaxa:
		if not highertaxonname in datasets.keys():
			continue
		cutoff=0
		if "cut-off" in datasets[highertaxonname].keys():
			cutoff=datasets[highertaxonname]["cut-off"]
		confidence=0
		if "confidence" in datasets[highertaxonname].keys():
			confidence=datasets[highertaxonname]["confidence"]
		minalignmentlength=0	
		if "min alignment length" in datasets[highertaxonname].keys():
			minalignmentlength=datasets[highertaxonname]["min alignment length"]		
#		if "sequence number" in datasets[highertaxonname].keys():
#			seqno=datasets[highertaxonname]["sequence number"]	
#		if "group number" in datasets[highertaxonname].keys():
#			groupno=datasets[highertaxonname]["group number"]	
		#if not ((seqno >0 and seqno < args.minseqno) or (groupno >0 and groupno < args.mingroupno)):	
		#if not (groupno < minGroupNo or seqno < minSeqNo):	
		if maxconfidence < confidence:
			maxconfidence =confidence
			bestcutoff=cutoff
			bestminalignmentlength=minalignmentlength
			besttaxon=highertaxonname	
	return bestcutoff,maxconfidence,besttaxon,bestminalignmentlength

def SaveBestCutoffsAsCutoffs(cutoffs,classificationdict,jsonoutputname,txtoutputname,problematicoutputname,problematicoutputname1):
	count=0
	total=0
	count1=0
	total1=0
	for rank in cutoffs.keys():
		datasets=cutoffs[rank]
		for datasetname in datasets.keys():
			dataset=datasets[datasetname]
			cutoff=dataset["cut-off"]
			confidence=dataset["confidence"]
			minalignmentlength=0
			if "min alignment length" in dataset.keys():
				minalignmentlength=dataset["min alignment length"]
			bestcutoff=cutoff
			maxconfidence=confidence
			bestminalignmentlength=minalignmentlength
			besttaxon=datasetname
			dataset["original cut-off"]=cutoff
			dataset["original confidence"]=confidence
			dataset["dataset with max confidence"]=datasetname
			dataset["original min alignment length"]=minalignmentlength
			if datasetname !="All":
				total=total +1	
			if datasetname in classificationdict.keys():
				classification=classificationdict[datasetname]["classification"]
				bestcutoff,maxconfidence,besttaxon,bestminalignmentlength=GetCutoffAndConfidence(rank,classification,cutoffs)					
			dataset["cut-off"]=bestcutoff
			dataset["confidence"]=maxconfidence
			dataset["dataset with max confidence"]=besttaxon
			dataset["min alignment length"]=bestminalignmentlength
	#save in json			
	with open(jsonoutputname,"w") as json_file:
		if sys.version_info[0] >= 3:
			json.dump(cutoffs,json_file,indent=2)
		else:
			json.dump(cutoffs,json_file,encoding='latin1',indent=2)
	#save as tab. format
	textfile=open(txtoutputname,"w")
	problematicfile=open(problematicoutputname,"w")
	problematicfile1=open(problematicoutputname1,"w")
	textfile.write("Rank\tDataset\toriginal cut-off\toriginal confidence\toriginal min alignment length\tDataset with max. confidence\tcut-off\tconfidence\tmin alignment length\tsequence number\tgroup number\tfasta filename\tclassification filename\n")
	problematicfile.write("Rank\tDataset\toriginal cut-off\toriginal confidence\toriginal min alignment length\tDataset with max. confidence\tcut-off\tconfidence\tmin alignment length\tsequence number\tgroup number\tfasta filename\tclassification filename\n")
	problematicfile1.write("Rank\tDataset\toriginal cut-off\toriginal confidence\toriginal min alignment length\tDataset with max. confidence\tcut-off\tconfidence\tmin alignment length\tsequence number\tgroup number\tfasta filename\tclassification filename\n")
	globaltotal=0
	globalcount=0
	for rank in cutoffs.keys():
		datasets=cutoffs[rank]
		#globalcutoff=0
		globalconfidence=0
		if "All" in datasets.keys():
			dataset=datasets["All"]
			#globalcutoff= dataset["cut-off"]
			globalconfidence=dataset["confidence"]
		for datasetname in datasets.keys():
			dataset=datasets[datasetname]
			cutoff=dataset["cut-off"]
			confidence=dataset["confidence"]
			minalignmentlength=dataset["min alignment length"]
			originalcutoff=dataset["original cut-off"]
			originalconfidence=dataset["original confidence"]
			originalminalignmentlength=dataset["original min alignment length"]
			highertaxon=dataset["dataset with max confidence"]
			SeqNo=dataset["sequence number"]
			GroupNo=dataset["group number"]
			fastafilename=dataset["fasta filename"]
			classificationfilename=dataset["classification filename"]
			#classificationposition=dataset["classification position"]
			textfile.write(rank+"\t" + datasetname + "\t"+str(originalcutoff)+"\t"+str(originalconfidence) + "\t" + str(originalminalignmentlength)+ "\t" + highertaxon +"\t" +str(cutoff)+"\t"+str(confidence)+ "\t" + str(minalignmentlength)+"\t"+str(SeqNo)+"\t"+str(GroupNo)+"\t"+fastafilename+"\t"+classificationfilename+"\n")
			if datasetname=="All":
				continue
			datasetname_rank=""
			if datasetname in classificationdict.keys():
				datasetname_rank=classificationdict[datasetname]["rank"]
			datasetname_level=GetLevel(datasetname_rank)
			level=GetLevel(rank)
			if datasetname_level==level - 1:
				total1=total1+1
			if confidence > originalconfidence and cutoff != originalcutoff:
				count=count+1
				problematicfile.write(rank+"\t" + datasetname + "\t"+str(originalcutoff)+"\t"+str(originalconfidence) + "\t" + str(originalminalignmentlength)+ "\t" + highertaxon +"\t" +str(cutoff)+"\t"+str(confidence)+ "\t" + str(minalignmentlength)+"\t"+str(SeqNo)+"\t"+str(GroupNo)+"\t"+fastafilename+"\t"+classificationfilename+"\n")
				if datasetname_level==level - 1:
					count1=count1+1
					problematicfile1.write(rank+"\t" + datasetname + "\t"+str(originalcutoff)+"\t"+str(originalconfidence) + "\t" + str(originalminalignmentlength)+ "\t" + highertaxon +"\t" +str(cutoff)+"\t"+str(confidence)+ "\t" + str(minalignmentlength)+"\t"+str(SeqNo)+"\t"+str(GroupNo)+"\t"+fastafilename+"\t"+classificationfilename+"\n")		
			if originalconfidence >= globalconfidence:
				globalcount=globalcount+1
			globaltotal=globaltotal+1				
	textfile.close()		
	problematicfile.close()
	problematicfile1.close()
	return count,total,count1,total1,globaltotal,globalcount

def SaveBestCutoffs(cutoffs,classificationdict,jsonoutputname,txtoutputname,problematicoutputname,problematicoutputname1):
	count=0
	total=0
	count1=0
	total1=0
	for rank in cutoffs.keys():
		datasets=cutoffs[rank]
		for datasetname in datasets.keys():
			dataset=datasets[datasetname]
			originalcutoff=dataset["cut-off"]
			originalconfidence=dataset["confidence"]
			originalminalignmentlength=0
			if "min alignment length" in dataset.keys():
				originalminalignmentlength=dataset["min alignment length"]
			bestcutoff=originalcutoff
			maxconfidence=originalconfidence
			bestminalignmentlength=originalminalignmentlength
			besttaxon=datasetname
			if datasetname !="All":
				total=total +1	
			if datasetname in classificationdict.keys():
				classification=classificationdict[datasetname]["classification"]
				bestcutoff,maxconfidence,besttaxon,bestminalignmentlength=GetCutoffAndConfidence(rank,classification,cutoffs)					
			dataset["best cut-off"]=bestcutoff
			dataset["max confidence"]=maxconfidence
			dataset["dataset with max confidence"]=besttaxon
			dataset["best min alignment length"]=bestminalignmentlength
	#save in json			
	with open(jsonoutputname,"w") as json_file:
		if sys.version_info[0] >= 3:
			json.dump(cutoffs,json_file,indent=2)
		else:
			json.dump(cutoffs,json_file,encoding='latin1',indent=2)
	#save as tab. format
	textfile=open(txtoutputname,"w")
	problematicfile=open(problematicoutputname,"w")
	problematicfile1=open(problematicoutputname1,"w")
	textfile.write("Rank\tDataset\tcut-off\tconfidence\tmin alignment length\tDataset with max. confidence\tbest cut-off\tmax confidence\tbest min alignment length\tsequence number\tgroup number\tfasta filename\tclassification filename\n")
	problematicfile.write("Rank\tDataset\tcut-off\tconfidence\tmin alignment length\tDataset with max. confidence\tbest cut-off\tmax confidence\tbest min alignment length\tsequence number\tgroup number\tfasta filename\tclassification filename\n")
	problematicfile1.write("Rank\tDataset\tcut-off\tconfidence\tmin alignment length\tDataset with max. confidence\tcut-off\tconfidence\tmin alignment length\tsequence number\tgroup number\tfasta filename\tclassification filename\n")
	globaltotal=0
	globalcount=0
	for rank in cutoffs.keys():
		datasets=cutoffs[rank]
		#globalcutoff=0
		globalconfidence=0
		if "All" in datasets.keys():
			dataset=datasets["All"]
			#globalcutoff= dataset["cut-off"]
			globalconfidence=dataset["confidence"]
		for datasetname in datasets.keys():
			dataset=datasets[datasetname]
			bestcutoff=dataset["best cut-off"]
			maxconfidence=dataset["max confidence"]
			bestminalignmentlength=dataset["best min alignment length"]
			originalcutoff=dataset["cut-off"]
			originalconfidence=dataset["confidence"]
			originalminalignmentlength=dataset["min alignment length"]
			highertaxon=dataset["dataset with max confidence"]
			SeqNo=dataset["sequence number"]
			GroupNo=dataset["group number"]
			fastafilename=dataset["fasta filename"]
			classificationfilename=dataset["classification filename"]
			#classificationposition=dataset["classification position"]
			textfile.write(rank+"\t" + datasetname + "\t"+str(originalcutoff)+"\t"+str(originalconfidence) + "\t" + str(originalminalignmentlength)+ "\t" + highertaxon +"\t" +str(bestcutoff)+"\t"+str(maxconfidence)+ "\t" + str(bestminalignmentlength)+"\t"+str(SeqNo)+"\t"+str(GroupNo)+"\t"+fastafilename+"\t"+classificationfilename+"\n")
			if datasetname=="All":
				continue
			datasetname_rank=""
			if datasetname in classificationdict.keys():
				datasetname_rank=classificationdict[datasetname]["rank"]
			datasetname_level=GetLevel(datasetname_rank)
			level=GetLevel(rank)
			if datasetname_level==level - 1:
				total1=total1+1
			if maxconfidence > originalconfidence and bestcutoff != originalcutoff:
				count=count+1
				problematicfile.write(rank+"\t" + datasetname + "\t"+str(originalcutoff)+"\t"+str(originalconfidence) + "\t" + str(originalminalignmentlength)+ "\t" + highertaxon +"\t" +str(bestcutoff)+"\t"+str(maxconfidence)+ "\t" + str(bestminalignmentlength)+"\t"+str(SeqNo)+"\t"+str(GroupNo)+"\t"+fastafilename+"\t"+classificationfilename+"\n")
				if datasetname_level==level - 1:
					count1=count1+1
					problematicfile1.write(rank+"\t" + datasetname + "\t"+str(originalcutoff)+"\t"+str(originalconfidence) + "\t" + str(originalminalignmentlength)+ "\t" + highertaxon +"\t" +str(bestcutoff)+"\t"+str(maxconfidence)+ "\t" + str(bestminalignmentlength)+"\t"+str(SeqNo)+"\t"+str(GroupNo)+"\t"+fastafilename+"\t"+classificationfilename+"\n")		
			if originalconfidence >= globalconfidence:
				globalcount=globalcount+1
			globaltotal=globaltotal+1				
	textfile.close()		
	problematicfile.close()
	problematicfile1.close()
	return count,total,count1,total1,globaltotal,globalcount
			
if __name__ == "__main__":
	if prefix=="" or prefix==None:
		prefix=GetBase(os.path.basename(cutoffsfilename))
	jsonoutputname=GetWorkingBase(prefix) + ".best.json"		
	txtoutputname=GetWorkingBase(prefix) + ".best.txt"	
	problematicoutputname=GetWorkingBase(prefix) + ".cutoffs.lowerconfidence.txt"
	problematicoutputname1=GetWorkingBase(prefix) + ".cutoffs.lowerconfidence.immediatehigherlevel.txt"	
	cutoffs={}
	if cutoffsfilename!="" and cutoffsfilename!=None:
		with open(cutoffsfilename) as cutoffsfile:
			cutoffs = json.load(cutoffsfile)
	classificationdict= LoadClassification(classificationfilename)
	if args.savebestcutoffsascutoffs=="yes":
		count,total,count1,total1,globaltotal,globalcount=SaveBestCutoffsAsCutoffs(cutoffs,classificationdict,jsonoutputname,txtoutputname,problematicoutputname,problematicoutputname1)
	else:
		count,total,count1,total1,globaltotal,globalcount=SaveBestCutoffs(cutoffs,classificationdict,jsonoutputname,txtoutputname,problematicoutputname,problematicoutputname1)
	if globaltotal >0:
		print("The best similarity cut-offs are saved in json and text format files " + jsonoutputname + " and " + txtoutputname + ".")
	print("The number of taxa having higher or equal prediction confidence than the global confidence: " + str(globalcount) + "/" + str(globaltotal) + " (" + str(round(globalcount*100/globaltotal,2)) +"%).")
	print("The number of taxa having the best cutoff: " + str(total-count) + "/" + str(total) + " (" + str(round((total-count)*100/total,2)) +"%).")
	print("The number of immedidate taxa having the best cutoff: " + str(total1-count1) + "/" + str(total1) + " (" + str(round((total1-count1)*100/total1,2)) +"%).")
	print("The local cut-offs for each taxon name at different taxonomic level are saved in file " + jsonoutputname + " and " + txtoutputname + ".")
	print("The taxa having lower prediction confidence and different cut-off than their higher taxa are saved in file " + problematicoutputname + ".")
	print("The immediate taxa having lower prediction confidence and different cut-off than their higher taxa are saved in file " + problematicoutputname1 + ".")
		
	


#!/usr/bin/env python
# FILE: mergesequences.py
# AUTHOR: Duong Vu
# CREATE DATE: 07 June 2020
import sys
import numpy as np
import os, argparse
from Bio import SeqIO
import json
import multiprocessing
nproc=multiprocessing.cpu_count()
#from keras.utils import np_utils

parser=argparse.ArgumentParser(prog='mergesequences.py', 
							   usage="%(prog)s [options] -i fastafiles -o output",
							   description='''Script that selects the sequences for of the given taxa. The taxon names in the taxa are separated by ","''',
							   epilog="""Written by Duong Vu duong.t.vu@gmail.com""",
   )

parser.add_argument('-i','--input', required=True, help='the fasta files to be merged')
parser.add_argument('-o','--out', required=True, help='The fasta output file containing the sequences with unique sequence ids') #optional

args=parser.parse_args()
output=args.out

fastafilenames=args.input.split(",")
seqids=[]
mergedrecords=[]
count=0
for fastafilename in fastafilenames:
	seqrecords=SeqIO.to_dict(SeqIO.parse(fastafilename, "fasta"))
	for seqid in seqrecords.keys():
		if not (seqid in seqids):
			if count >0 and seqid.startswith("K"):
				print(seqid)
			seqids.append(seqid)
			mergedrecords.append(seqrecords[seqid])
	count=count+1		
#save to file:
SeqIO.write(mergedrecords,output,"fasta")
print("All sequences are saved in " + output + ".")




#!/usr/bin/env python
# FILE: overview.py
# AUTHOR: Duong Vu
# CREATE DATE: 07 June 2020

import sys
import os, argparse
from Bio import SeqIO

parser=argparse.ArgumentParser(prog='overview.py',  
							   usage="%(prog)s [options] -i fastafile -c classificationfilename -out outputname",
							   description='''The script that summarizes the number of taxonomic groups of the sequences of the given fasta file. ''',
							   epilog="""Written by Duong Vu duong.t.vu@gmail.com""",
   )

parser.add_argument('-i','--input', default="", help='the fasta input file.')
parser.add_argument('-c','--classification', required=True, help='The taxonomic classification file.')
parser.add_argument('-o','--out', default="dnabarcoder", help='The output folder.')
parser.add_argument('-seqidpos','--sequenceidposition', type=int,default=0, help='the position of sequence id in the classification file.')

args=parser.parse_args()
fastafilename= args.input
classificationfilename= args.classification
outputpath=args.out
seqidpos=args.sequenceidposition

if not os.path.exists(outputpath):
	os.system("mkdir " + outputpath)

def GetWorkingBase(filename):
	basename=os.path.basename(filename)
	basename=basename[:-(len(basename)-basename.rindex("."))] 
	path=outputpath + "/" + basename
	return path

def ReportAtLevel(level,seqids,higherlevel,seqidpos):
	if level <0:
		return 0,0,0
	classificationfile = open(classificationfilename)
	next(classificationfile)
	taxa=[]
	seqNo=0
	count=0
	highertaxa={}
	for line in classificationfile:
		count=count+1
		words=line.split("\t")
		seqid=words[seqidpos].replace(".1","").replace(">","")
		if len(seqids) > 0 and (seqid not in seqids):
			continue
		if level >=  len(words):
			continue
		texts=line.split("\t")
		taxon=""
		highertaxon=""
		if level >-1:
			taxon=texts[level].rstrip()
		if higherlevel >-1:	
			highertaxon=texts[higherlevel].rstrip()
		if taxon=="" or taxon=="unidentified" or ("uncultured" in taxon) or ("_sp_" in taxon):
			continue
		seqNo=seqNo+1
		if not (taxon in taxa):
			taxa.append(taxon)
		if highertaxon!="":
			if not (highertaxon in highertaxa.keys()):
				highertaxa.setdefault(highertaxon,[])
			if not (taxon in highertaxa[highertaxon]):
				highertaxa[highertaxon].append(taxon)
	numberoftaxa=len(taxa)		
	classificationfile.close()
	return numberoftaxa,seqNo,count,highertaxa

def SaveOverview(rank,taxa,outputname):
	outfile=open(outputname,"w")
	outfile.write("Taxon\t"+ rank + " number\n")
	for taxon in taxa.keys():
		outfile.write(taxon + "\t" + str(len(taxa[taxon])) + "\n")
	outfile.close()

######MAIN
outputfilename=""
seqids=[]
if fastafilename != "":
	seqrecords = SeqIO.to_dict(SeqIO.parse(fastafilename, "fasta"))
	seqids=seqrecords.keys()
	outputfilename=GetWorkingBase(fastafilename) + ".overview"	
else:
	outputfilename=GetWorkingBase(classificationfilename) + ".overview"	
classificationfile=open(classificationfilename)
header=next(classificationfile)
classificationfile.close()
p_s=-1
p_g=-1
p_f=-1
p_o=-1
p_c=-1
p_p=-1
p_k=-1
words=header.split("\t")
i=0
for word in words:
	if word.rstrip().lower()=="species":
		p_s=i
	if word.rstrip().lower()=="genus":
		p_g=i
	if word.rstrip().lower()=="family":
		p_f=i	
	if word.rstrip().lower()=="order":
		p_o=i
	if word.rstrip().lower()=="class":
		p_c=i	
	if word.rstrip().lower()=="phylum":
		p_p=i	
	i=i+1           
outputfile=open(outputfilename,"w")
seqnumber,seqnumber,count,species=ReportAtLevel(seqidpos,seqids,p_s,seqidpos)
speciesnumber,speciesseqnumber,count,genera=ReportAtLevel(p_s,seqids,p_g,seqidpos)
genusnumber,genusseqnumber,count,families=ReportAtLevel(p_g,seqids,p_f,seqidpos)
familynumber,familyseqnumber,count,orders=ReportAtLevel(p_f,seqids,p_o,seqidpos)
ordernumber,orderseqnumber,count,classes=ReportAtLevel(p_o,seqids,p_c,seqidpos)
classnumber,classseqnumber,count,phyla=ReportAtLevel(p_c,seqids,p_p,seqidpos)
phylumnumber,phylumseqnumber,count,kingdoms=ReportAtLevel(p_p,seqids,p_k,seqidpos)
if len(seqids) >0:
	count=len(seqids)
print("Number of sequences: " + str(count))
print("Taxonomic level\tNumber of taxa\tNumber of sequences")
print("Species" + "\t" + str(speciesnumber) + "\t" + str(speciesseqnumber))
print("Genus" + "\t" + str(genusnumber) + "\t" + str(genusseqnumber))
print("Family" + "\t" + str(familynumber) + "\t" + str(familyseqnumber))
print("Order" + "\t" + str(ordernumber) + "\t" + str(orderseqnumber))
print("Class" + "\t" + str(classnumber) + "\t" + str(classseqnumber))
print("Phylum" + "\t" + str(phylumnumber) + "\t" + str(phylumseqnumber))	
outputfile.write("Number of sequences: " + str(count) + "\n")
outputfile.write("Taxonomic level\tNumber of taxa\tNumber of sequences\n")
outputfile.write("Species" + "\t" + str(speciesnumber) + "\t" + str(speciesseqnumber) + "\n")
outputfile.write("Genus" + "\t" + str(genusnumber) + "\t" + str(genusseqnumber) + "\n")
outputfile.write("Family" + "\t" + str(familynumber) + "\t" + str(familyseqnumber) + "\n")
outputfile.write("Order" + "\t" + str(ordernumber) + "\t" + str(orderseqnumber) + "\n")
outputfile.write("Class" + "\t" + str(classnumber) + "\t" + str(classseqnumber) + "\n")
outputfile.write("Phylum" + "\t" + str(phylumnumber) + "\t" + str(phylumseqnumber) + "\n")
outputfile.close()
print("The overview is saved in  file " + outputfilename + ".")
SaveOverview("sequence",species,outputfilename + ".species")
print("The overview at the species level is saved in  file " + outputfilename + ".species")
SaveOverview("species",genera,outputfilename + ".genus")
print("The overview at the genus level is saved in  file " + outputfilename + ".genus")
SaveOverview("genus",families,outputfilename + ".family")
print("The overview at the family level is saved in  file " + outputfilename + ".family")
SaveOverview("family",orders,outputfilename + ".order")
print("The overview at the order level is saved in  file " + outputfilename + ".order")
SaveOverview("order",classes,outputfilename + ".class")
print("The overview at the class level is saved in  file " + outputfilename + ".class")
SaveOverview("class",phyla,outputfilename + ".phylum")
print("The overview at the phylum level is saved in  file " + outputfilename + ".phylum")




#!/usr/bin/env python
# FILE: search.py
# AUTHOR: Duong Vu
# CREATE DATE: 07 June 2021
import sys
from sklearn.metrics import precision_recall_fscore_support
from sklearn.metrics import cohen_kappa_score
from sklearn.metrics import matthews_corrcoef
from sklearn.metrics import confusion_matrix
from sklearn.metrics import accuracy_score
if sys.version_info[0] >= 3:
	unicode = str
import os, argparse
from Bio import SeqIO
import json
import multiprocessing

nproc=multiprocessing.cpu_count()
#from keras.utils import np_utils

parser=argparse.ArgumentParser(prog='search.py',  
							   usage="%(prog)s [options] -i fastafile -r referencefastafile -ml minalignmentlength ",
							   description='''Script that classifies the sequences of the fasta files using BLAST with a cut-off value or the cut-off values given in the cutoffs file. ''',
							   epilog="""Written by Duong Vu duong.t.vu@gmail.com""",
   )

parser.add_argument('-i','--input', required=True, help='the fasta file to be classified.')
parser.add_argument('-r','--reference', required=True, help='the reference fasta file.')
parser.add_argument('-ml','--minalignmentlength', type=int, default=400, help='Minimum sequence alignment length required for BLAST. For short barcode sequences like ITS2 (ITS1) sequences, minalignmentlength should be set to smaller, 50 for instance.')
parser.add_argument('-o','--out', default="dnabarcoder", help='The output folder.')
parser.add_argument('-prefix','--prefix', help='the prefix of output filenames')

args=parser.parse_args()
testdataset= args.input
traindataset = args.reference
mincoverage = args.minalignmentlength

prefix=args.prefix
outputpath=args.out
if not os.path.exists(outputpath):
	os.system("mkdir " + outputpath)

def GetBase(filename):
	return filename[:-(len(filename)-filename.rindex("."))]
	
def GetWorkingBase(filename):
	basename=os.path.basename(filename)
	if "." in basename:
		basename=basename[:-(len(basename)-basename.rindex("."))] 
	path=outputpath + "/" + filename
	return path

def GetSeqIndex(seqname,seqrecords):
	i=0
	for seqrecord in seqrecords:
		if (seqname == seqrecord.id):
			return i
		i = i + 1
	return -1

def IndexSequences(filename):
	indexedfilename = GetBase(filename) + ".indexed.fasta"
	fastafile = open(filename)
	indexedfile = open(indexedfilename, "w")
	i=0
	for line in fastafile:
		if line.startswith('>'):
			indexedfile.write(">" + str(i) + "|" + line.rstrip()[1:] + "\n")
			i=i+1
		else:
			indexedfile.write(line)    
	fastafile.close()
	indexedfile.close()
	return indexedfilename

def ComputeBestBLASTscore(query,reference,mincoverage):
	indexed_query= IndexSequences(query)

	#load sequeces from the fasta files
	queryrecords = list(SeqIO.parse(indexed_query, "fasta"))
	#refrecords = list(SeqIO.parse(reference, "fasta"))

	bestscorelist =[0] * len(queryrecords)
	bestsimlist =[0] * len(queryrecords)
	bestcoveragelist =[0] * len(queryrecords)
	bestrefidlist = [""] * len(queryrecords)

	#blast
	makedbcommand = "makeblastdb -in " + reference + " -dbtype \'nucl\' " +  " -out db"
	print(makedbcommand)
	os.system(makedbcommand)
	#for short read
	blastcommand = "blastn -query " + indexed_query + " -db  db -task blastn-short -outfmt 6 -out out.txt -num_threads " + str(nproc)
	#for long read
	if mincoverage >=400:
		blastcommand = "blastn -query " + indexed_query + " -db  db -outfmt 6 -out out.txt -num_threads " + str(nproc)
	print(blastcommand)	
	os.system(blastcommand)
	
	#read blast output
	blastoutputfile = open("out.txt")
	refid = ""
	score=0
	queryid=""
	for line in blastoutputfile:
		words = line.split("\t")
		queryid=words[0]
		pos1 = int(words[6])
		pos2 = int(words[7])
		iden = float(words[2]) 
		sim=float(iden)/100
		coverage=abs(pos2-pos1)
		refid=words[1]
		score=sim
		if coverage < mincoverage:
				score=float(score * coverage)/mincoverage
		i = int(queryid.split("|")[0])		
		#if score > bestscorelist[i]:
		if score > bestscorelist[i] or (score == bestscorelist[i] and coverage > bestcoveragelist[i]):	
			bestscorelist[i]= score
			bestrefidlist[i]=refid
			bestsimlist[i]=sim
			bestcoveragelist[i]=coverage
	os.system("rm " + indexed_query)		
	os.system("rm out.txt")
	return bestrefidlist,bestscorelist,bestsimlist,bestcoveragelist

def SavePrediction(testseqIDs,bestscorelist,bestsimlist,bestcoveragelist,bestrefidlist,outputname):
	output=open(outputname,"w")
	output.write("SequenceID\tReferenceID\tBLAST score\tBLAST sim\tBLAST coverage\n")
	i=0
	for seqid in testseqIDs:
		output.write(seqid + "\t"  + bestrefidlist[i] + "\t" +  str(bestscorelist[i]) + "\t" + str(bestsimlist[i]) + "\t" + str(bestcoveragelist[i]) +"\n")
		i=i+1
	output.close()
	
##############################################################################
# MAIN
##############################################################################
path=sys.argv[0]
path=path[:-(len(path)-path.rindex("/")-1)]

#load ref seq records
refseqrecords = SeqIO.to_dict(SeqIO.parse(traindataset, "fasta"))

#load test seq records
testseqrecords = SeqIO.to_dict(SeqIO.parse(testdataset, "fasta"))

#search for a best match of a test sequence in a train dataset
bestmatchlist,bestscorelist,bestsimlist,bestcoveragelist=ComputeBestBLASTscore(testdataset,traindataset,mincoverage)

#Save prediction by searching 
if prefix=="" or prefix==None:
	prefix=GetBase(testdataset)
	if "/" in prefix:
		prefix=prefix[prefix.rindex("/")+1:]	
basename=GetBase(traindataset)
if "/" in basename:
	basename=basename[basename.rindex("/")+1:]		
reportfilename=GetWorkingBase(prefix) + "." + basename + "_BLAST.bestmatch"
SavePrediction(testseqrecords.keys(),bestscorelist,bestsimlist,bestcoveragelist,bestmatchlist,reportfilename)
print("The results are saved in file  " + reportfilename)




#!/usr/bin/env python
# FILE: getSeqLengthDistribution.py
# AUTHOR: Duong Vu
# CREATE DATE: 07 June 2020
import os
import sys, argparse
from Bio import SeqIO
import matplotlib.pyplot as plt
plt.rc('font',size=6)
import numpy as np

parser=argparse.ArgumentParser(prog='getSeqLengthDistribution.py',  
							   usage="%(prog)s [options] -i fastafile -l intervallength -out outputname",
							   description='''Script that computes the number of the sequences in increasing intervals of sequence lengths. ''',
							   epilog="""Written by Duong Vu duong.t.vu@gmail.com""",
   )

parser.add_argument('-i','--input', required=True, help='the fasta input file.')
parser.add_argument('-l','--intervallength', required=True, type=int, default=100, help='The length of the intervals.')
parser.add_argument('-o','--out',default="dnabarcoder", help='The output folder.')
parser.add_argument('-prefix','--prefix',default="", help='The prefix of the output files.')
parser.add_argument('-label','--label',default="", help='The label to display in the figure.')

args=parser.parse_args()
fastafilename= args.input
il= args.intervallength
outputpath=args.out
prefix=args.prefix
label=args.label

if not os.path.exists(outputpath):
	os.system("mkdir " + outputpath)
	
def GetBase(filename):
	if not ("." in filename):
		return filename
	return filename[:-(len(filename)-filename.rindex("."))] 

def GetWorkingBase(filename):
	basename=os.path.basename(filename)
	if "." in basename:
		basename=basename[:-(len(basename)-basename.rindex("."))] 
	path=outputpath + "/" + basename
	return path

def BarPlot(datasetname,labels,sums):
	x = np.arange(len(labels))  # the label locations
	#width = 0.35  # the width of the bars
	if len(labels) <50:
		fig, ax = plt.subplots(figsize=(3,2))
	else:
		fig, ax = plt.subplots(figsize=(6,2))
	ax.set_ylabel('Number of sequences')
	ax.set_title(datasetname + ": sequence length distribution")
	plt.bar(x,sums)
	ax.set_xticks(x)
	ax.set_xticklabels(labels,rotation=90)
	ax.legend()
	plt.tight_layout()
	plt.rcParams['font.size'] = 6.0
	plt.savefig(figoutput, dpi = 500)
	plt.show()
	
####MAIN
if prefix=="":
	prefix=GetBase(os.path.basename(fastafilename))
outputfilename=GetWorkingBase(prefix) + ".length.txt"
figoutput=GetBase(outputfilename)  + ".png"	
seqrecords = list(SeqIO.parse(fastafilename, "fasta"))
maxlength=0
minlength=0
for seqrecord in seqrecords:
	if len(seqrecord.seq) > maxlength:
		maxlength=len(seqrecord.seq)
	if minlength==0:
		minlength=len(seqrecord.seq) 
	elif len(seqrecord.seq)	< minlength:	
		minlength=len(seqrecord.seq)
outputfile=open(outputfilename,"w")
aver=0
n=int(maxlength/il) + 1
sums=[0] * n
for seqrecord in seqrecords:
	l=len(str(seqrecord.seq))
	i=int(l/il)
	sums[i]= sums[i] + 1
j=0
#save sequence length distribution
outputfile.write("Interval\tNumber of sequences\n")
intervals=[]
labels=[]
for i in range(0,n):
	outputfile.write("[" + str(j) + "," + str(j+il) + ")" + "\t" + str(sums[i]) + "\n")
	intervals.append(j)
	labels.append("[" + str(j) + "," + str(j+il) + ")")
	j=j+il
outputfile.close()
print("The minimum sequence length is " + str(minlength))
print("The maximum sequence length is " + str(maxlength))
print("The distribution and its figure are saved in files " + outputfilename + " and " + figoutput + ".")
#plot
if label=="":
	label=prefix
BarPlot(label,labels,sums)




#!/usr/bin/env python
# FILE: mergeCutoffs.py
# AUTHOR: Duong Vu
# CREATE DATE: 07 June 2020


import sys, argparse
import json
from copy import deepcopy

parser=argparse.ArgumentParser(prog='mergeCutoffs.py',  
							   usage="%(prog)s [options] -i listofdict -o outputname",
							   description='''The script that merges a list of dictionaries to one dictionary. ''',
							   epilog="""Written by Duong Vu duong.t.vu@gmail.com""",
   )

parser.add_argument('-i','--input', required=True, help='A list of dictionaries separated by commas.')
parser.add_argument('-o','--out', help='The merged dictionary.')


args=parser.parse_args()
dictionarylist= args.input
outputfilename=args.out


def SaveCutoffs(mergeddict,outputfilename):
	#save mergeddict		
	with open(outputfilename,"w") as json_file:
		if sys.version_info[0] >= 3:
			json.dump(mergeddict,json_file,indent=2)
		else:
			json.dump(mergeddict,json_file,encoding='latin1',indent=2)			
	#save as tab. format
	textoutput=outputfilename+".txt"
	textfile=open(textoutput,"w")
	textfile.write("Rank\tDataset\tcut-off\tconfidence\tsequence number\tgroup number\tmin alignment length\tfasta filename\tclassification filename\n")
	cutoff=0
	confidence=0
	SeqNo=0
	GroupNo=0
	fastafilename=""
	classificationfilename=""
	#classificationposition=-1
	for rank in mergeddict.keys():
		datasets=mergeddict[rank]
		for datasetname in datasets.keys():
			dataset=datasets[datasetname]
			cutoff=0
			if "cut-off" in dataset.keys():
				cutoff=dataset["cut-off"]
			confidence=0
			if "confidence" in dataset.keys():
				confidence=dataset["confidence"]
			SeqNo=0
			if "sequence number" in dataset.keys():
				SeqNo=dataset["sequence number"]
			GroupNo=0	
			if "group number" in dataset.keys():
				GroupNo=dataset["group number"]
			minalignmentlength=0
			if "min alignment length" in dataset.keys():
				minalignmentlength=dataset["min alignment length"]	
			fastafilename=""	
			if "fasta filename" in dataset.keys():	
				fastafilename=dataset["fasta filename"]
			classificationfilename=""	
			if 	"classification filename" in dataset.keys():
				classificationfilename=dataset["classification filename"]
			#classificationposition=""	
			#if 	"classification position" in dataset.keys():
			#	classificationposition=dataset["classification position"]
			textfile.write(rank+"\t" + datasetname + "\t"+str(cutoff)+"\t"+str(confidence)+"\t"+str(SeqNo)+"\t"+str(GroupNo)+"\t"+ str(minalignmentlength) + "\t" + fastafilename+"\t"+classificationfilename+"\n")
	textfile.close()
	print("The outputs are saved in " + outputfilename + " and " + textoutput + ".")
####MAIN####
dictionaries=[]
if "," in dictionarylist:
	dictionaries=dictionarylist.split(",")
else:
	dictionaries.append(dictionarylist)	
mergeddict={}	
for dictionaryname in dictionaries:
	cutoffdict={}
	#load classes
	with open(dictionaryname,encoding='latin1') as json_file:
		cutoffdict = json.load(json_file)
	if cutoffdict!={}:	
		for rank in cutoffdict.keys():
			if not (rank in mergeddict.keys()):
				mergeddict.setdefault(rank,{})
			mergeddatasets=mergeddict[rank]	
			datasets=cutoffdict[rank]
			for datasetname in datasets.keys():
				dataset=datasets[datasetname]
				confidence=0
				if "confidence" in dataset.keys():
					confidence=float(dataset["confidence"])
				if not (datasetname in mergeddatasets.keys()):
					mergeddatasets.setdefault(datasetname,{})
				mergeddataset=mergeddatasets[datasetname]
				mergedconfidence=0
				if "confidence" in mergeddataset.keys():
					mergedconfidence=float(mergeddataset["confidence"])
				if mergedconfidence < confidence:
					mergeddataset=dataset.copy()
				mergeddatasets[datasetname]=mergeddataset	
SaveCutoffs(mergeddict,outputfilename)





#!/usr/bin/env python
# FILE: overview.py
# AUTHOR: Duong Vu
# CREATE DATE: 07 June 2020

import sys
import os, argparse
from Bio import SeqIO

parser=argparse.ArgumentParser(prog='updateTaxonomicClassification.py',  
							   usage="%(prog)s [options] -i classificationfilename -t taxon -u newtaxon -o outputname",
							   description='''The script that filter the classification for the sequences of the given fasta file. ''',
							   epilog="""Written by Duong Vu duong.t.vu@gmail.com""",
   )

parser.add_argument('-i','--input', required=True, help='The taxonomic classification file.')
parser.add_argument('-t','--taxon', required=True, help='The current taxon.')
parser.add_argument('-u','--newtaxon', required=True, help='The updated taxon.')
parser.add_argument('-o','--out', help='The output filename. If this is not given then the original file will be updateted.')

args=parser.parse_args()
classificationfilename= args.input
taxon= args.taxon
newtaxon= args.newtaxon
outputfilename=args.out

def GetBase(filename):
	return filename[:-(len(filename)-filename.rindex("."))]  

if outputfilename==None:
	outputfilename=GetBase(classificationfilename) + ".taxaupdated.out"
classificationfile=open(classificationfilename)
header=next(classificationfile)
outputfile=open(outputfilename,"w")
outputfile.write(header)
for line in classificationfile:
	texts=line.split("\t")
	newline=""
	for text in texts:
		if text==taxon:
			text=newtaxon
		newline=newline + text + "\t"	
	newline=newline[:-1]
	outputfile.write(newline)
classificationfile.close()
outputfile.close()
if args.out==None:
	os.system("cp " + outputfilename + " " + classificationfilename)
	os.system("rm " + outputfilename)
	outputfilename=classificationfilename
print("The taxa is updated in  file " + outputfilename  + ".")



#!/usr/bin/env python
# FILE: verify.py
# AUTHOR: Duong Vu
# CREATE DATE: 07 June 2021
#from sklearn.model_selection import StratifiedKFold
#from sklearn.preprocessing import StandardScaler
import sys
if sys.version_info[0] >= 3:
	unicode = str
import os, argparse
#from sklearn.metrics import precision_recall_fscore_support
#from sklearn.metrics import cohen_kappa_score
#from sklearn.metrics import matthews_corrcoef
#from sklearn.metrics import confusion_matrix
#from sklearn.metrics import accuracy_score
#import json
from Bio import SeqIO
from Bio import Phylo
import pylab
import random
import multiprocessing
nproc=multiprocessing.cpu_count()

parser=argparse.ArgumentParser(prog='verify.py',  
							   usage="%(prog)s [options] -i classified file -f the fasta file -r referencefastafilename -c classificationfile -o output",
							   description='''Script that assigns the classified sequences of the prediction file to their BLAST best match based on the given cutoffs.''',
							   epilog="""Written by Duong Vu duong.t.vu@gmail.com""",
   )

parser.add_argument('-i','--input', required=True, help='the classified file')
parser.add_argument('-f','--fasta', required=True, help='the fasta file')
parser.add_argument('-r','--reference', required=True, help='the reference fasta file')
parser.add_argument('-o','--out', default="dnabarcoder", help='The output folder.')
parser.add_argument('-c','--classification', default="", help='the classification file in tab. format.')
parser.add_argument('-m','--maxseqno', type=int, default=50, help='Maximum number of the sequences of the predicted taxon name from the classification file will be selected for the comparison to find the best match. If it is not given, all the sequences will be selected.')
parser.add_argument('-rank','--classificationrank', default="", help='the classification rank')
parser.add_argument('-redo','--redo', default="", help='the classification rank')
parser.add_argument('-prefix','--prefix', help='the prefix of output filenames')

args=parser.parse_args()
predictionfilename=args.input
fastafilename= args.fasta
referencefastafilename= args.reference
classificationfilename=args.classification
maxseqno=args.maxseqno
prefix=args.prefix
verifyingrank=args.classificationrank
redo=args.redo
outputpath=args.out

if not os.path.exists(outputpath):
	os.system("mkdir " + outputpath)
nproc=multiprocessing.cpu_count()

def GetBase(filename):
	return filename[:-(len(filename)-filename.rindex("."))]
	
def GetWorkingBase(filename):
	basename=os.path.basename(filename)
	if "." in basename:
		basename=basename[:-(len(basename)-basename.rindex("."))] 
	path=outputpath + "/" + filename
	return path

def GetTaxonomicClassification(level,header,texts):
	classification=""
	p_s=len(texts)
	p_g=len(texts)
	p_f=len(texts)
	p_o=len(texts)
	p_c=len(texts)
	p_p=len(texts)
	p_k=len(texts)
	i=0
	for text in header.split("\t"):
		text=text.rstrip()
		if text.lower()=="species":
			p_s=i
		elif text.lower()=="genus":
			p_g=i	
		elif text.lower()=="family":
			p_f=i	
		elif text.lower()=="order":
			p_o=i	
		elif text.lower()=="class":
			p_c=i	
		elif text.lower()=="phylum":
			p_p=i	
		elif text.lower()=="kingdom":
			p_k=i	
		i=i+1 
	species="unidentified"
	genus="unidentified"
	family="unidentified"
	order="unidentified" 
	bioclass="unidentified"
	phylum="unidentified"
	kingdom="unidentified"
	if p_s< len(texts):
		species=texts[p_s].rstrip()
		if species=="" or ("unculture" in species) or ("_sp_" in species) or ("unidentified" in species) or ("environmental" in species.lower()) or ("(" in species):
			species="unidentified"
		elif not (" " in species or "_" in species):
			species="unidentified"
		species=species.replace(" ","_")
	if p_g< len(texts):
		genus=texts[p_g].rstrip()
		if genus=="" or ("unculture" in genus) or ("unidentified" in genus):
			genus="unidentified"
	if p_f< len(texts):
		family=texts[p_f].rstrip()
		if family=="" or ("unculture" in family) or ("unidentified" in family):
			family="unidentified"
	if p_o< len(texts):
		order=texts[p_o].rstrip()
		if order=="" or ("unculture" in order) or ("unidentified" in order):
			order="unidentified"
	if p_c< len(texts):
		bioclass=texts[p_c].rstrip()
		if bioclass=="" or ("unculture" in bioclass) or ("unidentified" in bioclass):
			bioclass="unidentified"
	if p_p< len(texts):
		phylum=texts[p_p].rstrip()
		if phylum=="" or ("unculture" in phylum) or ("unidentified" in phylum):
			phylum="unidentified"
	if p_k< len(texts):
		kingdom=texts[p_k].rstrip()
		if kingdom=="" or ("unculture" in kingdom) or ("unidentified" in kingdom):
			kingdom="unidentified"
	taxonname=""
	rank=""
	if level <7 and kingdom!="unidentified":
		taxonname=kingdom
		rank="kingdom"
		classification="k__" + kingdom +";p__unidentified;c__unidentified;o__unidentified;f__unidentified;g__unidentified;s__unidentified"	
	if level <6 and phylum!="unidentified":
		taxonname=phylum
		rank="phylum"
		classification="k__" + kingdom +";p__"+phylum +";c__unidentified;o__unidentified;f__unidentified;g__unidentified;s__unidentified"
	if level <5 and bioclass!="unidentified":
		taxonname=bioclass
		rank="class"
		classification="k__" + kingdom +";p__"+phylum +";c__"+bioclass+";o__unidentified;f__unidentified;g__unidentified;s__unidentified"
	if level <4 and order!="unidentified":
		taxonname=order
		rank="order"
		classification="k__" + kingdom +";p__"+phylum +";c__"+bioclass +";o__"+ order + ";f__unidentified;g__unidentified;s__unidentified"
	if level <3 and family!="unidentified":
		taxonname=family
		rank="family"
		classification="k__" + kingdom +";p__"+phylum +";c__"+bioclass +";o__"+ order+";f__"+family +";g__unidentified;s__unidentified"
	if level <2 and genus!="unidentified":
		taxonname=genus
		rank="genus"
		classification="k__" + kingdom +";p__"+phylum +";c__"+bioclass +";o__"+ order+";f__"+family + ";g__"+ genus + ";s__unidentified"
	if level <1 and species!="unidentified":
		taxonname=species
		rank="species"
		classification="k__" + kingdom +";p__"+phylum +";c__"+bioclass +";o__"+ order+";f__"+family + ";g__"+ genus+";s__"+species 	
	return classification,taxonname,rank

def LoadClassification(seqrecords,classificationfilename):
	classificationdict={}
	classes={}
	if classificationfilename == "":
		return {},{}
	classificationfile= open(classificationfilename)
	header=next(classificationfile)
	for line in classificationfile:
		texts=line.split("\t")
		seqid = texts[0].replace(">","").rstrip()
		classification,taxonname,rank=GetTaxonomicClassification(0,header,texts)
		if classification!="":
			classificationdict.setdefault(seqid,{})
			classificationdict[seqid]["classification"]=classification
			classificationdict[seqid]["taxonname"]=taxonname
			classificationdict[seqid]["rank"]=rank	
		if seqid in seqrecords.keys():
			taxonnames=classification.split(";")
			for taxonname in taxonnames:
				if taxonname=="":
					continue
				taxonname=taxonname.split("__")[1].replace("_"," ")
				if taxonname=="unidentified" or taxonname=="":
					continue
				if not taxonname in classes.keys():
					classes.setdefault(taxonname,{})
				classes[taxonname][seqid]=seqrecords[seqid]
	classificationfile.close()	
	return classificationdict,classes

def LoadPrediction(predictionfilename):
	predictiondict={}
	p_id=-1
	p_l=-1
	p_pl=-1
	p_c=-1
	p_p=-1
	p_r=-1
	p_bs=-1
	p_s=-1
	p_co=-1
	p_rank=-1
	p_v=-1
	p_t=-1
	p_cutoff=-1
	p_confidence=-1
	p_numberofrefsequences=-1
	p_branchlength=-1
	p_maxbranchlength=-1
	p_averagebranchlength=-1
	predictionfile=open(predictionfilename)
	headers=next(predictionfile)
	i=0
	for header in headers.rstrip().split("\t"):
		if ("sequenceid" in header.lower()) or ("sequence_id" in header.lower()) or ("sequence id" in header.lower()) :
			p_id=i
		if "given label" in header.lower():
			p_l=i
		if "prediction" in header.lower():
			p_pl=i
		if "full classification" in header.lower():
			p_c=i
		if "probability" in header.lower():
			p_p=i
		if ("referenceid" in header.lower()) or ("reference_id" in header.lower()) or ("reference id" in header.lower()) :
			p_r=i
		if "score" in header.lower():
			p_bs=i	
		if "sim" in header.lower():
			p_s=i
		if "coverage" in header.lower():
			p_co=i	
		if "rank" in header.lower():
			p_rank=i
		if "cut-off" in header.lower():
			p_cutoff=i	
		if "confidence" in header.lower():
			p_confidence=i		
		if "verified label" in header.lower():
			p_v=i
		if "tree filename" in header.lower():
			p_t=i	
		if "number of reference sequences" in header.lower():
			p_numberofrefsequences=i
		if "branch length" in header.lower():
			p_branchlength=i
		if "max branch length" in header.lower():
			p_maxbranchlength=i	
		if "average branch length" in header.lower():
			p_averagebranchlength=i	
		i=i+1	
	for line in predictionfile:
		texts=line.rstrip().split("\t")
		if p_id >=0 and p_id < len(texts):
			seqid=texts[p_id]
			predictiondict.setdefault(seqid,{})
		else:
			continue
		label=""
		if p_l >=0 and p_l < len(texts):
			label=texts[p_l]
		predictiondict[seqid]["givenlabel"]=label
		predlabel=""
		if p_pl >=0 and p_pl < len(texts):
			predlabel=texts[p_pl]
		predictiondict[seqid]["predlabel"]=predlabel
		pred_classification=""
		if p_c >=0 and p_c < len(texts):
			pred_classification=texts[p_c]
		predictiondict[seqid]["classification"]=pred_classification
		proba=1
		if p_p >=0 and p_p <len(texts):
			proba=float(texts[p_p])
		predictiondict[seqid]["proba"]=proba
		refid=""
		if p_r>0 and p_r<len(texts):
			refid=texts[p_r]
		predictiondict[seqid]["refid"]=refid
		score=0
		if p_bs>0 and p_bs<len(texts):
			score=float(texts[p_bs])
		predictiondict[seqid]["score"]=score
		sim=0
		if p_s>0 and p_s<len(texts):
			sim=float(texts[p_s])
		predictiondict[seqid]["sim"]=sim
		cov=""
		if p_co>0 and p_co<len(texts):
			cov=float(texts[p_co])
		predictiondict[seqid]["coverage"]=cov
		rank=""
		if p_rank>0 and p_rank<len(texts):
			rank=texts[p_rank]
		predictiondict[seqid]["rank"]=rank
		cutoff=0
		if p_cutoff>0 and p_cutoff<len(texts):
			cutoff=texts[p_cutoff]
		predictiondict[seqid]["cutoff"]=cutoff
		confidence=0
		if p_confidence>0 and p_confidence<len(texts):
			confidence=texts[p_confidence]
		predictiondict[seqid]["confidence"]=confidence
		verifiedlabel=""
		if p_v>0 and p_v<len(texts):
			verifiedlabel=texts[p_v]
		predictiondict[seqid]["verifiedlabel"]=verifiedlabel
		treefilename=""
		if p_t>0 and p_t<len(texts):
			treefilename=texts[p_t]
		predictiondict[seqid]["treefilename"]=treefilename
		numberofrefsequences=0
		if p_numberofrefsequences>0 and p_numberofrefsequences<len(texts):
			numberofrefsequences=texts[p_numberofrefsequences]
		predictiondict[seqid]["numberofrefsequences"]=numberofrefsequences
		branchlength=0
		if p_branchlength>0 and p_branchlength<len(texts):
			branchlength=texts[p_branchlength]
		predictiondict[seqid]["branchlength"]=branchlength
		maxbranchlength=0
		if p_maxbranchlength>0 and p_maxbranchlength<len(texts):
			maxbranchlength=texts[p_maxbranchlength]
		predictiondict[seqid]["maxbranchlength"]=maxbranchlength
		averagebranchlength=0
		if p_averagebranchlength>0 and p_averagebranchlength<len(texts):
			averagebranchlength=texts[p_averagebranchlength]
		predictiondict[seqid]["averagebranchlength"]=averagebranchlength
		
	return predictiondict

def GetLevel(rank):
	level=-1
	if rank=="species":	
		level=6
	elif rank=="genus":	
		level=5
	elif rank=="family":	
		level=4
	elif rank=="order":	
		level=3
	elif rank=="class":	
		level=2
	elif rank=="phylum":
		level=1
	elif rank=="kingdom":
		level=0	
	return level

def lookup_by_names(tree):
	names = {}
	for clade in tree.find_clades():
		if clade.name:
			if clade.name in names:
				raise ValueError("Duplicate key: %s" % clade.name)
			names[clade.name] = clade
	return names

def verifyBasedOnBranchLengths(seqid,treefilename):
	tree = Phylo.read(treefilename, "newick")
	names = lookup_by_names(tree)
	max_length=0
	length=0
	average=0
	n=0
	for name in names:
		clade=names[name]
		if name==seqid:
			length=clade.branch_length
		else:
			average=average + clade.branch_length
			n=n+1
			if clade.branch_length > max_length:
				max_length=clade.branch_length
	verified=(length<=max_length) and (len(names) >=3)
	if n>0:
		average=round(average/n,2)
	return verified,length,max_length,average

def PrintTree(treefilename,redo):
	svgfilename=GetWorkingBase(fastafilename) + ".tree.svg"
	if (not os.path.exists(svgfilename)) or redo!="":
		tree = Phylo.read(treefilename, "newick")
		Phylo.draw(tree,do_show=False)	
		#Phylo.draw_ascii(tree,do_show=False)	
		pylab.savefig(svgfilename,format='svg', bbox_inches='tight', dpi=300)
		print("A iq-tree and its svg file are saved in " + treefilename + " and " + svgfilename + ".")	
	
def CreateTree(fastafilename,redo):
	alignmentfilename =  GetBase(fastafilename) + ".aligned.fas"
	if (not os.path.exists(alignmentfilename)) or redo!="":
		#make the alignment
		if not os.path.exists(alignmentfilename):
			command="clustalo -i " + fastafilename + " -o " + alignmentfilename
			os.system(command)
	treefilename=GetBase(fastafilename) + ".aligned.fas.treefile"
	if (not os.path.exists(treefilename)) or redo!="":
		#make tree
		command="iqtree -pers 0.2 -n 500 -s " + alignmentfilename
		#command="iqtree -s " + alignmentfilename
		os.system(command)
	#print tree
	PrintTree(treefilename,redo)
	return treefilename	

def CreateFastaFile(seqrecord,taxonname,classeswithsequences,numberofrefsequences,maxseqno,redo):
	if not os.path.exists(outputpath + "/trees"):
		os.system("mkdir " + outputpath + "/trees")
	if sys.version_info[0] < 3:
		taxonname=unicode(taxonname,errors='ignore')
	newfastafilename=outputpath + "/trees/" + seqrecord.id.replace("|","_") + ".fasta"
	if not os.path.exists(newfastafilename) or redo!="":
		if taxonname in classeswithsequences.keys():
			seqrecords=[]
			seqrecords.append(seqrecord)
			sequences=classeswithsequences[taxonname]
			numberofrefsequences=len(sequences.keys())
			if len(sequences) >=2:#only make tree of more than 3 sequences
				if (maxseqno >0) and (len(sequences) > maxseqno):
					#select randomly 100 sequences to compare
					selectedlist=random.sample(range(0, len(sequences)), k=maxseqno)
					for i in selectedlist:
						seqids=list(sequences.keys())
						sequenceid=seqids[i]
						if sequenceid != seqrecord.id:
							seqrecords.append(sequences[sequenceid])
				else:	
					for sequenceid in sequences.keys():
						if sequenceid != seqrecord.id:
							seqrecords.append(sequences[sequenceid])
				if len(seqrecords) >=3:			
					SeqIO.write(seqrecords,newfastafilename,"fasta")
				else:
					newfastafilename=""
			else:
				newfastafilename=""
	if numberofrefsequences==0 and os.path.exists(newfastafilename):
		records=SeqIO.to_dict(SeqIO.parse(fastafilename, "fasta"))
		numberofrefsequences=len(records)-1
	return newfastafilename,numberofrefsequences
	
def Verify(seqrecords,predictiondict,refclasses,maxseqno,verifyingrank):
	count=0
	notree_count=0
	total=0
	#create Fasta files
	for seqid in predictiondict.keys():
		predictedname=predictiondict[seqid]["predlabel"]
		rank=predictiondict[seqid]["rank"]
		treefilename=predictiondict[seqid]["treefilename"]
		numberofrefsequences=predictiondict[seqid]["numberofrefsequences"]
		branchlength=predictiondict[seqid]["branchlength"]
		maxbranchlength=predictiondict[seqid]["maxbranchlength"]
		averagebranchlength=predictiondict[seqid]["averagebranchlength"]
		#if sys.version_info[0] < 3:
			#predictedname=unicode(predictedname,'latin1')
		verifiedlabel=""
		if (verifyingrank=="" or (verifyingrank!="" and rank==verifyingrank)):
			#only predict when the tree file name does not exist
			if predictedname in refclasses.keys() and seqid in seqrecords.keys():
				total=total+1
				seqrecord=seqrecords[seqid]
				if treefilename=="" or (redo !=""):
					treefastafilename,numberofrefsequences=CreateFastaFile(seqrecord,predictedname,refclasses,numberofrefsequences,maxseqno,redo)
					if os.path.exists(treefastafilename):
						treefilename= CreateTree(treefastafilename,redo)
				verified=False		
				if 	os.path.exists(treefilename):	
					verified,branchlength,maxbranchlength,averagebrachlength=verifyBasedOnBranchLengths(seqid,treefilename)
				else:
					notree_count=notree_count+1
				if verified==True:
					verifiedlabel=predictedname
					count=count+1
		predictiondict[seqid]["verifiedlabel"]=verifiedlabel
		predictiondict[seqid]["treefilename"]=treefilename		
		predictiondict[seqid]["numberofrefsequences"]=numberofrefsequences
		predictiondict[seqid]["branchlength"]=branchlength
		predictiondict[seqid]["maxbranchlength"]=maxbranchlength
		predictiondict[seqid]["averagebranchlength"]=averagebranchlength
	return count,notree_count,total

def SaveVerification(predictiondict,output,notverifiedoutput,classificationfilename):
	classificationreportfile=open(classificationreportfilename,"w")
	outputfile=open(output,"w")
	notverifiedoutputfile=open(notverifiedoutput,"w")
	notverifiedoutputfile.write("#SequenceID\tGiven label\tPrediction\tFull classification\tProbability\tRank\tCut-off\tConfidence\tReferenceID\tBLAST score\tBLAST sim\tBLAST coverage\tVerified label\tTree filename\tNumber of reference sequences\tBranch length\tMax branch length\tAverage branch length\n")
	outputfile.write("#SequenceID\tGiven label\tPrediction\tFull classification\tProbability\tRank\tCut-off\tConfidence\tReferenceID\tBLAST score\tBLAST sim\tBLAST coverage\tVerified label\tTree filename\tNumber of reference sequences\tBranch length\tMax branch length\tAverage branch length\n")
	classificationreportfile.write("SequenceID\tReferenceID\tkingdom\tphylum\tclass\torder\tfamily\tgenus\tspecies\trank\tscore\tcutoff\tconfidence\n")
	for seqid in predictiondict.keys():
		prediction=predictiondict[seqid]
		treefilename=prediction["treefilename"]
		predlabel=prediction["predlabel"]
		givenlabel=prediction["givenlabel"]
		classification=prediction["classification"]
		rank=prediction["rank"]
		bestscore=prediction["score"]
		cutoff=prediction["cutoff"]
		confidence=prediction["confidence"]
		proba=prediction["proba"]
		refid=prediction["refid"]
		sim=prediction["sim"]
		coverage=prediction["coverage"]
		verifiedlabel=prediction["verifiedlabel"]
		numberofrefsequences=prediction["numberofrefsequences"]
		branchlength=prediction["branchlength"]
		maxbranchlength=prediction["maxbranchlength"]
		averagebranchlength=prediction["averagebranchlength"]
		if verifiedlabel!="" and verifiedlabel!="unidentified":			
			outputfile.write(seqid + "\t" + givenlabel + "\t"  + predlabel + "\t"+ classification + "\t" + str(proba) + "\t" + rank + "\t" + str(cutoff) + "\t" + str(confidence) + "\t" + refid + "\t" + str(bestscore) + "\t" + str(sim) + "\t" + str(coverage) + "\t" + verifiedlabel + "\t" + treefilename + "\t" + str(numberofrefsequences) + "\t" + str(branchlength) + "\t" + str(maxbranchlength) + "\t" + str(averagebranchlength) + "\n")			
			cleanclassification=classification.replace("k__","").replace("p__","").replace("c__","")	.replace("o__","").replace("f__","").replace("g__","").replace("s__","").replace("_"," ")
			if refid==seqid:
				refid=""
			classificationreportfile.write(seqid + "\t" + refid + "\t" + cleanclassification.replace(";","\t") + "\t" + rank + "\t" + str(bestscore) + "\t" + str(cutoff) + "\t" + str(confidence) + "\n")
		elif verifyingrank!="" and verifyingrank==rank:
			notverifiedoutputfile.write(seqid + "\t" + givenlabel + "\t"  + predlabel + "\t"+ classification + "\t" + str(proba) + "\t" + rank + "\t" + str(cutoff) + "\t" + str(confidence) + "\t" + refid + "\t" + str(bestscore) + "\t" + str(sim) + "\t" + str(coverage) + "\t" + verifiedlabel + "\t" + treefilename + "\t" + str(numberofrefsequences) + "\t" + str(branchlength) + "\t" + str(maxbranchlength) + "\t" + str(averagebranchlength) + "\n")			
	classificationreportfile.close()
	outputfile.close()	
	notverifiedoutputfile.close()	
		

def LoadClassificationForKronaReport(classificationfilename):
	classificationdict={}
	classificationfile= open(classificationfilename)
	next(classificationfile)
	for line in classificationfile:
		if line.startswith("#"):
			continue 
		texts=line.split("\t")
		classification=texts[3]
		classification=classification.replace("k__","").replace("p__","").replace("c__","").replace("o__","").replace("f__","").replace("g__","").replace("s__","")
		if ";" in classification:
			texts=classification.split(";")
			classification=texts[0]
			if classification=="":
				classification="unidentified"
			for i in range(1,len(texts)):
				text=texts[i]
				if text=="":
					text="unidentified"
				classification=classification + "\t" + text	
		if classification in classificationdict.keys():
			classificationdict[classification]=classificationdict[classification] + 1
		else:
			classificationdict.setdefault(classification, 1)
	classificationfile.close()	
	return classificationdict

def KronaPieCharts(classification,kronareport,kronahtml):
	kronareportfile=open(kronareport,"w")
	for classname in classification.keys():
		kronareportfile.write(str(classification[classname]) + "\t" + classname + "\n")
	kronareportfile.close()	
	#create kronahtml
	command="ImportText.pl " + kronareport + " -o " + kronahtml
	#print(command)
	os.system(command)
	os.system("firefox " + kronahtml) 
if __name__ == "__main__":
	if prefix=="" or prefix==None:
		prefix=GetBase(predictionfilename)
		if "/" in prefix:
			prefix=prefix[prefix.rindex("/")+1:]	
	outputname=GetWorkingBase(prefix) + ".verified"
	notverifiedoutputname=GetWorkingBase(prefix) + ".notverified"
	classificationreportfilename=GetWorkingBase(prefix) + ".verified.classification"
	if outputname==predictionfilename:
		outputname=outputname+".verified"
	if verifyingrank!="":
		outputname=GetWorkingBase(prefix) + "." + verifyingrank + ".verified"
		notverifiedoutputname=GetWorkingBase(prefix) + "." + verifyingrank + ".notverified"
		classificationreportfilename=GetWorkingBase(prefix) + "." + verifyingrank + ".verified.classification"
#	if outputname==predictionfilename:
#		outputname=outputname+".verified"
	#load prediction
	predictiondict=LoadPrediction(predictionfilename)	
	#load sequences
	seqrecords={}
	if os.path.exists(fastafilename):
		seqrecords=SeqIO.to_dict(SeqIO.parse(fastafilename, "fasta"))
	#load reference sequences:	
	refseqrecords={}
	if os.path.exists(referencefastafilename):
		refseqrecords=SeqIO.to_dict(SeqIO.parse(referencefastafilename, "fasta"))
	refclassificationdict,refclasses= LoadClassification(refseqrecords,classificationfilename)
	#verifying...	
	count,notree_count,total=Verify(seqrecords,predictiondict,refclasses,maxseqno,verifyingrank)
	if total >0:
		print("Number of classified sequences: " + str(total))
		print("Number of verified sequences: " + str(count) + "(" + str(round(count*100/total,2)) + " %).")
		print("Number of sequences without alignment: " + str(notree_count) + "(" + str(round(notree_count*100/total,2)) + " %).")
		print("Number of sequences with failt verification: " + str(total - count - notree_count) + "(" + str(round((total - count - notree_count)*100/total,2)) + " %).")
	#print("The results are saved in file  " + outputname)
	SaveVerification(predictiondict,outputname,notverifiedoutputname,classificationfilename)
	print("The results are saved in file  " + outputname + ", " + notverifiedoutputname + " and " + classificationreportfilename + ".")
	#making krona report
	kronareport = GetBase(outputname) + ".krona.report"
	kronahtml=GetBase(kronareport) + ".html"
	classificationdict= LoadClassificationForKronaReport(outputname)
	KronaPieCharts(classificationdict,kronareport,kronahtml)
	print("The krona report and html are saved in files " + kronareport + " and " + kronahtml + ".") 



#!/usr/bin/env python
# FILE: overview.py
# AUTHOR: Duong Vu
# CREATE DATE: 07 June 2020

import sys
import os, argparse
from Bio import SeqIO

parser=argparse.ArgumentParser(prog='filterTaxonomicClassification.py',  
							   usage="%(prog)s [options] -i fastafile/subclassificationfile -c classificationfilename -o outputname",
							   description='''The script that filter the classification for the sequence ids given in the input file. ''',
							   epilog="""Written by Duong Vu duong.t.vu@gmail.com""",
   )

parser.add_argument('-i','--input', required=True, help='the fasta input file.')
parser.add_argument('-c','--classification', required=True, help='The taxonomic classification file.')
parser.add_argument('-seqidpos','--sequenceidposition', type=int, default=0, help='the position of the sequence ids in the classification file.')
parser.add_argument('-selectedpositions','--selectedpositions', default="", help='the selected positions for loading data from the classification file.')

parser.add_argument('-o','--out', default="", help='The output filename.')

args=parser.parse_args()
inputfilename= args.input
classificationfilename= args.classification
outputfilename=args.out
seqidpos= args.sequenceidposition
selectedpositions=args.selectedpositions


def GetBase(filename):
	if not ("." in filename):
		return filename
	return filename[:-(len(filename)-filename.rindex("."))]

#if not os.path.exists(outputpath):
#	os.system("mkdir " + outputpath)
	
#def GetWorkingBase(filename):
#	basename=os.path.basename(filename)
#	basename=basename[:-(len(basename)-basename.rindex("."))] 
#	path=outputpath + "/" + basename
#	return path

######MAIN
#load sequence ids
seqids =[]
inputfile=open(inputfilename)
for line in inputfile:
	if line.startswith(">"):
		seqid=line.rstrip().split(" ")[0]
		seqid=seqid.replace(">","")
		seqids.append(seqid)
	elif "\t" in line:
		seqid=line.split("\t")[seqidpos]
		seqids.append(seqid.rstrip())	
positionlist=[]
if selectedpositions !="":
	positions=selectedpositions.split(",")
	for position in positions:
		if position=="":
			continue
		positionlist.append(int(position))
		
if outputfilename==None or outputfilename=="":
	outputfilename=GetBase(inputfilename) + ".classification"		
taxafilename=GetBase(outputfilename) + ".taxonomic.classification"	
classificationfile=None
if sys.version_info[0] ==3 :
	classificationfile=open(classificationfilename, encoding='latin-1')
else:
	classificationfile=open(classificationfilename)
header=next(classificationfile)
if len(positionlist)>0:
	texts=header.split("\t")
	header=""
	for pos in positionlist:
		header=header + texts[pos].rstrip() + "\t" 
	header=header[:-1] + "\n"	
p_s=0
p_g=0
p_f=0
p_o=0
p_c=0
p_p=0
p_k=0
words=header.split("\t")
i=0
for word in words:
   if word.rstrip().lower()=="species":
	   p_s=i
   if word.rstrip().lower()=="genus":
	   p_g=i
   if word.rstrip().lower()=="family":
	   p_f=i	
   if word.rstrip().lower()=="order":
	   p_o=i
   if word.rstrip().lower()=="class":
	   p_c=i	
   if word.rstrip().lower()=="phylum":
	   p_p=i
   if word.rstrip().lower()=="kingdom":
	   p_k=i	        
   i=i+1
outputfile=open(outputfilename,"w")	            	   
taxafile=open(taxafilename,"w")
if (p_s+p_g+p_g+p_f+p_o+p_c+p_p+p_k)>0:
	taxafile.write("#Sequence ID\tkingdom\tphylum\tclass\torder\tfamily\tgenus\tspecies\n")
outputfile.write(header)
for line in classificationfile:
	words=line.split("\t")	   
	seqid=words[seqidpos].rstrip()			
	if not seqid in seqids:
		continue
	words=line.split("\t")
	species=""
	if p_s>0 and p_s<len(words):
		species=words[p_s].rstrip()
	genus=""
	if p_g>0 and p_g<len(words):
		genus=words[p_g].rstrip()	
	family=""
	if p_f>0 and p_f<len(words):
		family=words[p_f].rstrip()	
	order=""
	if p_o>0 and p_o<len(words):
		order=words[p_o].rstrip()	
	bioclass=""
	if p_c>0 and p_c<len(words):
		bioclass=words[p_c].rstrip()	
	phylum=""
	if p_p>0 and p_p<len(words):
		phylum=words[p_p].rstrip()
	if p_k>0 and p_k<len(words):
		kingdom=words[p_k].rstrip()	
	if (p_s+p_g+p_g+p_f+p_o+p_c+p_p+p_k)>0:	
		taxclassification=seqid + "\t" + kingdom + "\t" + phylum + "\t" + bioclass + "\t" + order + "\t" + family + "\t" + genus + "\t" + species  + "\n"
		taxafile.write(taxclassification)
	classification=line	
	if len(positionlist)>0:
		texts=line.split("\t")
		classification=""
		for pos in positionlist:
			classification=classification + texts[pos].rstrip() + "\t" 
		classification=classification[:-1] + "\n"	
	outputfile.write(classification)
classificationfile.close()
taxafile.close()
outputfile.close()
if (p_s+p_g+p_g+p_f+p_o+p_c+p_p+p_k)>0:	
	print("The classifications are saved in  files " + outputfilename + " and " + taxafilename + ".")
else:
	os.system("rm " + taxafilename)
	print("The classifications are saved in  files " + outputfilename + ".")
	



#!/usr/bin/env python
# FILE: addclassificationtosequenceheaders.py
# AUTHOR: Duong Vu
# CREATE DATE: 07 June 2019
import sys, argparse
if sys.version_info[0] >= 3:
	unicode = str
import numpy as np
import os
from Bio import SeqIO
import multiprocessing
nproc=multiprocessing.cpu_count()

parser=argparse.ArgumentParser(prog='splitclassificationtosequenceheaders.py',  
							   usage="%(prog)s [options] -i fastafile -c classificationfilename -o output",
							   description='''Script that remove classification from sequence headers. ''',
							   epilog="""Written by Duong Vu duong.t.vu@gmail.com""",
   )

parser.add_argument('-i','--input', required=True, help='the fasta file to be clustered.')
parser.add_argument('-o','--out', help='The fasta filename with sequence headers containing classification.') 
parser.add_argument('-c','--classification', required=True, help='the output file containing classification file in tab. format.')
parser.add_argument('-p','--classificationposition', default="", help='the classification positions of the features to kept for sequence headers.')

args=parser.parse_args()
fastafilename= args.input
classificationfilename=args.classification
output=args.out
classificationpos=args.classificationposition

#fastafilename=sys.argv[1]
#classificationfilename=sys.argv[2]
#output=sys.argv[3]

def GetBase(filename):
	return filename[:-(len(filename)-filename.rindex("."))]

#####main###
poslist=[]
if "," in classificationpos:
	texts=args.classificationpos.split(",")
	for t in texts:
		poslist.append(int(t))
elif classificationpos!="":
	poslist.append(int(classificationpos))
if len(poslist)==0:
	poslist.append(0)
#seqrecords=list(SeqIO.parse(fastafilename, "fasta"))
classificationfile=open(classificationfilename,"w")
outputfile=open(output,"w")
fastafile=open(fastafilename)
write=True
seqids=[]
for line in fastafile:
	if line.startswith(">"):
		write=True
		header=line.rstrip().replace(">","")
		classification=header
		seqid=header
		if "|" in header:
			classfication=header.replace("|","\t")
			texts=header.split("|")
			header=""
			seqid=texts[0]
			for pos in poslist:
				if header=="":
					seqid=texts[pos].replace(" ","")
					header=header + seqid + "|"
				else:
					header=header + texts[pos] + "|"
			classification=header.replace("|","\t")
			for i in range(0,len(texts)):
				if i not in poslist:
					classification=classification + texts[i] + "\t"
			header=header[:-1]	
			classification=classification[:-1]
		if not (seqid in seqids):
			seqids.append(seqid)
			classificationfile.write(classification + "\n")	
			outputfile.write(">" + header + "\n")	
		else:
			write=False
	else:
		if write==True:		
			outputfile.write(line)
outputfile.close()		
classificationfile.close()
fastafile.close()



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""
Created on Fri May 28 12:28:09 2021

@author: Duong Vu
"""
import sys
if sys.version_info[0] >= 3:
	unicode = str
import os, argparse
import multiprocessing
from Bio import SeqIO

nproc=multiprocessing.cpu_count()

parser=argparse.ArgumentParser(prog='computeSim.py',  
							   usage="%(prog)s [options] -i fastafile -o output",
							   description='''Script that computes BLAST similarity matrix for sequences of a fasta file. ''',
							   epilog="""Written by Duong Vu duong.t.vu@gmail.com""",
   )

parser.add_argument('-i','--input', required=True, help='the fasta file')
parser.add_argument('-o','--out',default="dnabarcoder", help='The output folder.') 
parser.add_argument('-ml','--minalignmentlength', type=int, default=400, help='Minimum sequence alignment length required for BLAST. For short barcode sequences like ITS2 (ITS1) sequences, minalignmentlength should be set to smaller, 50 for instance.')
parser.add_argument('-ms','--minsim', type=float, default=0, help='The minimum similarity score that will be saved for the output.')

args=parser.parse_args()
fastafilename= args.input
mincoverage=args.minalignmentlength
minsim=args.minsim
outputpath=args.out

if not os.path.exists(outputpath):
	os.system("mkdir " + outputpath)	
	
def GetBase(filename):
	return filename[:-(len(filename)-filename.rindex("."))] 

def GetWorkingBase(filename):
	basename=os.path.basename(filename)
	basename=basename[:-(len(basename)-basename.rindex("."))] 
	path=outputpath + "/" + basename
	return path

def SaveSim(simmatrix,simfilename,ms):
	simfile=open(simfilename,"w")
	for i in simmatrix.keys():
		for j in simmatrix[i].keys():
			if simmatrix[i][j] >= ms:
				simfile.write(str(i) + " " + str(j) + " " + str(simmatrix[i][j]) + "\n")
	simfile.close()

def ComputeSim(fastafilename,mincoverage,minsim):
	#scorematrix = [[0 for x in range(len(seqrecords))] for y in range(len(seqrecords))] 
#	simfilename=GetBase(fastafilename) + ".sim"
#	simfile=open(simfilename,"w")
	#blast
	makedbcommand = "makeblastdb -in " + fastafilename + " -dbtype \'nucl\' " +  " -out db"
	os.system(makedbcommand)
	blastcommand = "blastn -query " + fastafilename + " -db  db -task blastn-short -outfmt 6 -out out.txt -num_threads " + str(nproc)
	if mincoverage >=400:
		blastcommand = "blastn -query " + fastafilename + " -db  db -outfmt 6 -out out.txt -num_threads " + str(nproc)
	os.system(blastcommand)
	
	seqrecords=SeqIO.to_dict(SeqIO.parse(fastafilename, "fasta"))
	simmatrix={}
	for seqid in seqrecords.keys():
		simmatrix.setdefault(seqid,{})
		simmatrix[seqid][seqid]=1
	
	#read blast output
	blastoutputfile = open("out.txt")
	for line in blastoutputfile:
		if line.rstrip()=="":
			continue
		words = line.split("\t")
#		queryid = words[0].rstrip()
#		refid = words[1].rstrip()
		i = words[0].rstrip()
		j = words[1].rstrip()
#		i = seqids.index(queryid)
#		j = seqids.index(refid)
		pos1 = int(words[6])
		pos2 = int(words[7])
		iden = float(words[2]) 
		sim=float(iden)/100
		coverage=abs(pos2-pos1)
		score=sim
		if coverage < mincoverage:
			score=float(score * coverage)/mincoverage
		if j in simmatrix[i].keys():
			if simmatrix[i][j] < score:
				simmatrix[i][j]=round(score,4)
				simmatrix[j][i]=round(score,4)
		else:
			simmatrix[i][j]=round(score,4)
			simmatrix[j][i]=round(score,4)
#		if scorematrix[i][j] < score:
#			scorematrix[i][j]=score
#			scorematrix[j][i]=score
	os.system("rm out.txt")
	return simmatrix

if __name__ == "__main__":
	#load sequences
	base = sys.argv[0][0: sys.argv[0].rindex("/")+1]
	simmatrix={}
	print("Computing similarity matrix using BLAST..")
	simmatrix=ComputeSim(fastafilename,mincoverage,minsim)
	output=GetWorkingBase(fastafilename) + ".sim"
	#save the simmatrix
	SaveSim(simmatrix,output,minsim)
	print("The similarity file is saved in " + output + ".")		
			
		



#!/usr/bin/env python
# FILE: assignment2funguildformat.py
# AUTHOR: Duong Vu
# CREATE DATE: 07 June 2021
import sys
if sys.version_info[0] >= 3:
	unicode = str
import os, argparse
from Bio import SeqIO

parser=argparse.ArgumentParser(prog='assignment2funguildformat.py',  
							   usage="%(prog)s [options] -i assignment/classificationfile",
							   description='''Script that creates a funguild format file from an assignment/classification file''',
							   epilog="""Written by Duong Vu duong.t.vu@gmail.com""",
   )

parser.add_argument('-i','--assignmentfilename', required=True, help='the classification filename 1')
parser.add_argument('-o','--out',default="dnabarcoder", help='The output folder.')

args=parser.parse_args()
assignmentfilename= args.assignmentfilename
outputpath=args.out
if not os.path.exists(outputpath):
	os.system("mkdir " + outputpath)	

def GetBase(filename):
	return filename[:-(len(filename)-filename.rindex("."))]

def GetWorkingBase(filename):
	basename=os.path.basename(filename)
	basename=basename[:-(len(basename)-basename.rindex("."))] 
	path=outputpath + "/" + basename
	return path

def GetClassification(fullclassification):
	kingdom=""
	phylum=""
	bioclass=""
	order=""
	family=""
	genus=""
	species=""
	texts=fullclassification.split(";")
	for text in texts:
		if text.startswith("k__"):
			kingdom=text.replace("k__","")
		if text.startswith("p__"):
			phylum=text.replace("p__","")	
		if text.startswith("c__"):
			bioclass=text.replace("c__","")	
		if text.startswith("o__"):
			order=text.replace("o__","")	
		if text.startswith("f__"):
			family=text.replace("f__","")	
		if text.startswith("g__"):
			genus=text.replace("g__","")	
		if text.startswith("s__"):
			species=text.replace("s__","")	
	return kingdom,phylum,bioclass,order,family,genus,species

def Level2Number(level):
	number=0
	if level.lower()=="species":
		number=7
	elif level.lower()=="genus":
		number=6
	elif level.lower()=="family":
		number=5	
	elif level.lower()=="order":
		number=4
	elif level.lower()=="class":
		number=3
	elif level.lower()=="phylum":
		number=2
	elif level.lower()=="kingdom":
		number=1	
	return number
#######MAIN#######################
#outputclassificationclean= GetWorkingBase(assignmentfilename) + ".classification"
#outputfilename= GetWorkingBase(assignmentfilename) + ".funguild"
outputclassificationclean= assignmentfilename + ".clean"
outputfilename=assignmentfilename + ".funguild"
outputfile=open(outputfilename,"w")
outputclassificationcleanfile=open(outputclassificationclean,"w")
inputfile=open(assignmentfilename)
header=inputfile.readline()
words=header.split("\t")
p_s=-1
p_g=-1
p_f=-1
p_o=-1
p_c=-1
p_p=-1
p_k=-1
p_score=-1
p_level=-1
p_fullclassification=-1
i=0
for word in words:
	word=word.rstrip()
	if word.lower()=="kingdom":
		p_k=i
	if word.lower()=="phylum":
		p_p=i	
	if word.lower()=="class":
		p_c=i	
	if word.lower()=="order":
		p_o=i	
	if word.lower()=="family":
		p_f=i	
	if word.lower()=="genus":
		p_g=i	
	if word.lower()=="species":
		p_s=i
	if "score" in word.lower():
		p_score=i	
	if word.lower()=="level":
		p_level=i
	if word.lower()=="rank":
		p_level=i	
	if word.lower()=="full classification":
		p_fullclassification=i		
	i=i+1
outputfile.write("OTU_ID\ttaxonomy\n")
outputclassificationcleanfile.write("OTU_ID\tkingdom\tphylum\tclass\torder\tfamily\tgenus\tspecies\tscore\n")
for line in inputfile:
	kingdom=""
	phylum=""
	bioclass=""
	order=""
	family=""
	genus=""
	species=""
	words=line.split("\t")
	seqid=words[0].split("|")[0]
	score=1
	number=8
	if p_score>-1:
		score=float(words[p_score])
		if score >1:
			score=score/100
	if p_level >-1:
		number=Level2Number(words[p_level])	
	fullclassification=""
	if p_fullclassification > -1:
		fullclassification=words[p_fullclassification]
		kingdom,phylum,bioclass,order,family,genus,species=GetClassification(fullclassification)
	else:
		if p_k< len(words) and p_k>-1:
			kingdom=words[p_k].rstrip()
		if kingdom=="":
			kingdom="Fungi"		   
		if p_p < len(words) and p_p>-1 and number >=2:
			phylum=words[p_p].rstrip()
		if phylum=="" or  "unculture" in phylum:
			phylum="unidentified"	
		if p_c<len(words) and p_c>-1 and number >=3:
			bioclass=words[p_c].rstrip()
		if bioclass=="" or  "unculture" in bioclass:	
			bioclass="unidentified"
		if p_o<len(words) and p_o>-1 and number >=4:
			order=words[p_o].rstrip()
		if order=="" or  "unculture" in order:
			order="unidentified"
		if p_f<len(words) and p_f>-1 and number >=5:
			family=words[p_f].rstrip()
		if family=="" or  "unculture" in family:
			family="unidentified"
		if p_g<len(words) and p_g > -1 and number >=6:
			genus=words[p_g].rstrip()
		if genus=="" or ("unculture" in genus):
			genus="unidentified"
		if p_s<len(words) and p_s >-1 and number >=7:
			species=(words[p_s]).rstrip()
		if species=="" or ("unculture" in species) or "_sp_" in species:
			species="unidentified"
		if " " in species:
			species=species.replace(" ","_")
		fullclassification= "k__" + kingdom + ";" + "p__" + phylum + ";" + "c__" + bioclass + ";" + "o__" + order + ";" + "f__" + family + ";" + "g__" + genus + ";" + "s__" + species + ";"
	outputclassificationcleanfile.write(seqid +"\t" + kingdom + "\t" + phylum + "\t" + bioclass +"\t" + order + "\t" + family + "\t" + genus + "\t" + species.replace("_"," ") + "\t" + str(score) + "\n") 
	outputfile.write(seqid + "\t" + str(round(score*100,2)) + "%|" + fullclassification + "\n")
inputfile.close()
outputfile.close()
outputclassificationcleanfile.close()
print("The outputs are saved in file: " + outputfilename + " and " + outputclassificationclean)



from setuptools import setup, find_packages


setup(
    name='dnabarcoder',
    version='0.0.2',
    license='Apache License 2.0',
    author="Duong Vu",
    author_email='duong.t.vu@gmail.com',
    package_dir={'': 'src'},
    packages=['dnabarcoder','dnabarcoder.analysis','dnabarcoder.prediction','dnabarcoder.visualization','dnabarcoder.aidscripts','dnabarcoder.classification'],
    url='https://github.com/vuthuyduong/dnabarcoder',
    keywords='dnabarcoder',
    install_requires=[
          'scikit-learn',
	  'scipy==1.2.1',
      ],
    include_package_data=True,
)
